%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%% This program estimates %%%%%%%%
%%%%%%%%%% the model's hyperparameters %%%
%%%%%%%%%% and computes posteriors %%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%%%%   SET UP MATLAB %%%%%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
       
        %Clear;
        clear all;
        rng(12345)
        warning ('on','all');
        
        %Switches;
        load_data=0;
        estimate_model=0;
        display=0;
        posteriors=0;

        %Working directory;
        directory  = '';

        %Model options;
        model='3'; %this identifies which value-added specification to use;
        vers = '05122016'; %date of first step estimates from VAM_reformat_and_fillin
        grade=6;
        sfx=1; %this determines whether to include sector effects;
        known_fs=0; %this determines whether to treat the first stage as fixed;
        pooled=1; %this determines whether to allow temporal variation in VA parameters;
        restrict_rho=1; %this determines whether to set the correlation of idiosyncratic shocks to 0;
            restrict_sigma=1;
        start_fixed=1; %this determines whether the posterior mode max problem starts at fixed FS estimates;

        %Optimization options;
        options_fmin=struct('Display','iter','MaxFunEvals',100000,'MaxIter',500,'GradObj','off','TolFun',1e-6);     
        R_init=10;
        R_WT=1000;
        R_final=500;
        
        %Parameters of DGP;
        T=1+7*(1-pooled);
        J=50;
        max_years=1;
        if grade==7;
            J=46;
            max_years=2;
        end;
        if grade==8;
            J=43;
            max_years=3;
        end;
        
        %Format model name;
        savename = ['results_' model];
        if sfx==1;
            savename=[savename '_sfx'];
        end;
        if covs==1;
            savename=[savename '_covs'];
        end;
        if pooled==1;
            savename=[savename '_pooled'];
        end;
        if known_fs==1;
            savename=[savename '_known_fs'];
        end;

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%%%%%% LOAD DATA %%%%%%%%%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
        if load_data==1;

            %Covariates;

               sects=dlmread([directory 'ests_reformat' vers  '\VAM_sects.csv'],',',1,1);
               if pooled~=1;
                   sects=dlmread([ directory 'ests_reformat' vers  '\VAM_sects_long.csv' ] );
               end;   
               if grade==7;
                   sects=dlmread([ directory  'ests_reformat' vers  '\VAM_sectsg7.csv' ] );
               end;
               if grade==8;
                   sects=dlmread([ directory 'ests_reformat' vers  '\VAM_sectsg8.csv' ] );
               end;

                charter=sects(:,2);
                pilot=sects(:,3);
                
                J = length(charter);

               P=[ones(J,1) charter pilot];
               if sfx==0;
                   P=ones(J,1);
               end;
               K=length(P(1,:));


            %Results storage;

                rho_hat=zeros(J,T);
                alpha_hat=zeros(J,T);
                pi_hat=zeros(J,J,T);
                Q=zeros(J,T);
                A=zeros(J,T);
                V_alpha=zeros(J,T);
                V_rho=zeros(J,T);
                chol_V=zeros(J*(2+J),J*(2+J),T);
                t=1;

            %Estimates;

                if pooled==1;

                        data_temp = dlmread(['ests_reformat' vers  '\VAM_ests_m' model ' _test2.csv'  ],',',1);
                        Q_temp= dlmread(['ests_reformat' vers  '\Q.csv'  ],',',1);
                        data_temp(data_temp==-999)=0;
                        
                        if grade==7;
                            Q_temp=dlmread(['Qg7.csv'])';
                        end;
                        if grade==8;
                            Q_temp=dlmread(['Qg8.csv'])';
                        end;
                        
                        V_temp = dlmread(['ests_reformat' vers  '\VAM_vce_m' model ' _test2.csv'  ],',',1,1);
                        
                        alpha_temp=data_temp(1:J);
                        A_temp=1-(alpha_temp==-999);
                        Q_temp=Q_temp((J+1):(2*J));
                        rho_temp=data_temp((J+1):(2*J));
                        pi_long_temp=data_temp(((2*J)+1):end);
                        pi_temp=zeros(J,J);
                        count=1;
                        for l=1:J;
                           for j=1:J;
                               pi_temp(j,l)=pi_long_temp(count);
                               count=count+1;
                           end;
                        end;
                        Q(:,t)=Q_temp;
                        A(:,t)=A_temp;
                        alpha_hat(:,t)=alpha_temp.*A_temp;
                        rho_hat(:,t)=rho_temp.*Q_temp;
                        pi_temp=pi_temp.*repmat(Q_temp',[J 1]);
                        pi_hat(:,:,t)=pi_temp.*repmat(A_temp,[1 J]);
                        for j=1:J;
                            if A(j,t)==0;
                                V_temp(j,j)=0.1;
                            end;
                            if Q(j,t)==0;
                                V_temp(J+j,J+j)=0.1;
                            end;
                        end
                        chol_V(:,:,t)=chol(V_temp);
                        V_alpha(:,t)=diag(V_temp(1:J,1:J));
                        V_rho(:,t)=diag(V_temp((J+1):(2*J),(J+1):(2*J)));
                        t=t+1;
                        pi=pi_hat;
                end;
              
                if pooled==0;

                    %Loop over years;
                    for y=2007:2014;
                        y
                        %Bring in data;
                        data_temp=dlmread(['ests_reformat' vers  '\VAM_ests_m' model '_test2_year' num2str(y) '.csv'])';
                        Q_temp=dlmread(['ests_reformat' vers  '\Q_year' num2str(y) '.csv'])';
                        V_temp=dlmread(['ests_reformat' vers  '\VAM_vce_m' model '_test2_year' num2str(y) '.csv']);

                        %Format elements;
                        alpha_temp=data_temp(1:J);
                        A_temp=1-(alpha_temp==-999);
                        Q_temp=Q_temp((J+1):(2*J));
                        rho_temp=data_temp((J+1):(2*J));
                        pi_long_temp=data_temp(((2*J)+1):end);
                        pi_temp=zeros(J,J);
                        count=1;
                        for l=1:J;
                           for j=1:J;
                               pi_temp(j,l)=pi_long_temp(count);
                               count=count+1;
                           end;
                        end;

                        %Store results;
                        Q(:,t)=Q_temp;
                        A(:,t)=A_temp;
                        alpha_hat(:,t)=alpha_temp.*A_temp;
                        rho_hat(:,t)=rho_temp.*Q_temp;
                        pi_temp=pi_temp.*repmat(Q_temp',[J 1]);
                        pi_hat(:,:,t)=pi_temp.*repmat(A_temp,[1 J]);

                        for j=1:J;
                            if A(j,t)==0;
                                V_temp(j,j)=0.1;
                            end;
                            if Q(j,t)==0;
                                V_temp(J+j,J+j)=0.1;
                            end;
                        end;
                        chol_V(:,:,t)=chol(V_temp);
                        V_alpha(:,t)=diag(V_temp(1:J,1:J));
                        V_rho(:,t)=diag(V_temp((J+1):(2*J),(J+1):(2*J)));
                        t=t+1;

                    end;
                    pi=pi_hat; 

                end;
                                                
               
              
            %Construct cell arrays to pass to optimizer;

                data_cell_smd={alpha_hat,rho_hat,pi_hat,P,A,Q};
                V_cell={V_alpha,V_rho};

          %%%%%%%%%%%%%INITIALIZING VALUES %%%%%%%%%%%%%

            %Initial values;

                %Means;
                theta_P= zeros(4,size(P,2));

                if sfx==0;
                    theta_P=theta_P(:,9);
                end;
                if known_fs==1;
                    theta_P=theta_P(1:2,:);
                end;

                %Variances;
                sigma_beta=0.3;
                sigma_b=0.3;
                sigma_delta=1;
                sigma_chi=2;
                rho_beta_b=0;
                rho_beta_delta=-0.2;
                rho_beta_chi=-0.2;
                rho_b_delta=0.2;
                rho_b_chi=0.2;
                rho_delta_chi=0;
                sigma_nu=1;
                sigma_beta_tilde=0.2;
                sigma_b_tilde=0.2;
                rho_tilde=0;
                Sigma=[sigma_beta*[sigma_beta sigma_b*rho_beta_b sigma_delta*rho_beta_delta sigma_chi*rho_beta_chi]; ...
                       sigma_b*[sigma_beta*rho_beta_b sigma_b sigma_delta*rho_b_delta sigma_chi*rho_b_chi]; ...
                       sigma_delta*[sigma_beta*rho_beta_delta sigma_b*rho_b_delta sigma_delta sigma_chi*rho_delta_chi]; ...
                       sigma_chi*[sigma_beta*rho_beta_chi sigma_b*rho_b_chi sigma_delta*rho_delta_chi sigma_chi]];
                if known_fs==1;
                    Sigma=[sigma_beta*[sigma_beta sigma_b*rho_beta_b]; ...
                       sigma_b*[sigma_beta*rho_beta_b sigma_b]]; 
                end;
                   chol_Sigma=chol(Sigma);

                %Any lotto effect;
                theta_Q=0.12;

                %%Initial guess;
                theta_true=[theta_Q theta_P(:)' log(diag(chol_Sigma))' chol_Sigma(1,2:end)]';
                if known_fs~=1;
                    theta_true=[theta_true' chol_Sigma(2,3:end) chol_Sigma(3,4) log(sigma_nu)]';
                end;
                if restrict_sigma~=1;
                    theta_true=[theta_true' log(sigma_beta_tilde) log(sigma_b_tilde)]';
                end;
                if restrict_rho~=1;
                    theta_true=[theta_true' 0.5*log((1+rho_tilde)/(1-rho_tilde))]';
                end;
                theta_0=theta_true;
                if rand_start==1;
                    theta_0=randn(length(theta_0),1);
                end;
                if continue_est==1;
                    load(savename);
                    theta_0=cell2mat(output(2));
                end;


                
        end;
       
  
        
        
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%%%%%% ESTIMATE MODEL %%%%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

        if estimate_model==1;

            %%%%%%%%%% DRAW SIMULATIONS %%%%%%%%%%%%%%%;

                    %Initial;
                    sims_omega_j=randn(J,4,R_init);
                    sims_omega_tilde=randn(J,2,T,R_init);
                    sims_nu=randn(J,J,T,R_init);
                    sims_e=randn(J*(2+J),T,R_init);
                    e_sim=zeros(J*(2+J),T,R_init);
                    for t=1:T;
                        for r=1:R_init;
                            e_sim(:,t,r)=((sims_e(:,t,r)')*chol_V(:,:,t))';
                        end;
                    end;
                    sims_cell_init={sims_omega_j,sims_omega_tilde,sims_nu,e_sim};

                        %Weighting matrix;
                        sims_omega_j=randn(J,4,R_WT);
                        sims_omega_tilde=randn(J,2,T,R_WT);
                        sims_nu=randn(J,J,T,R_WT);
                        sims_e=randn(J*(2+J),T,R_WT);
                        e_sim=zeros(J*(2+J),T,R_WT);
                        for t=1:T;
                            for r=1:R_WT;
                                e_sim(:,t,r)=((sims_e(:,t,r)')*chol_V(:,:,t))';
                            end;
                        end;
                        sims_cell_WT={sims_omega_j,sims_omega_tilde,sims_nu,e_sim};

                        %Final sims;
                        sims_omega_j=randn(J,4,R_final);
                        sims_omega_tilde=randn(J,2,T,R_final);
                        sims_nu=randn(J,J,T,R_final);
                        sims_e=randn(J*(2+J),T,R_final);
                        e_sim=zeros(J*(2+J),T,R_final);
                        for t=1:T;
                            for r=1:R_final;
                                e_sim(:,t,r)=((sims_e(:,t,r)')*chol_V(:,:,t))';
                            end;
                        end;
                        sims_cell_final={sims_omega_j,sims_omega_tilde,sims_nu,e_sim};

                    end;


             %%%%%%%%%%%%%%%% ESTIMATE MODEL %%%%%%%%%%%       


                        %Step 0: Run fsolve to get starting values;
                        WT=get_weight(theta_0,data_cell_smd,V_cell,sims_cell_WT,0,theta_true,restrict_rho,restrict_sigma,known_fs,max_years);
                        [theta_hat_0_A Q_hat]=fsolve(@(x)smd(x,WT,data_cell_smd,V_cell,sims_cell_init,0,theta_true,restrict_rho,restrict_sigma,known_fs,max_years),theta_0,options_fmin);
                        [theta_0 theta_hat_0_A]
                       WT=get_weight(theta_hat_0_A,data_cell_smd,V_cell,sims_cell_WT,0,theta_true,restrict_rho,restrict_sigma,known_fs,max_years);
                       [theta_hat_0_B Q_hat]=fsolve(@(x)smd(x,WT,data_cell_smd,V_cell,sims_cell_init,0,theta_true,restrict_rho,restrict_sigma,known_fs,max_years),theta_hat_0_A,options_fmin);
                       [theta_0 theta_hat_0_A theta_hat_0_B]
                       WT=get_weight(theta_hat_0_B,data_cell_smd,V_cell,sims_cell_WT,1,theta_true,restrict_rho,restrict_sigma,known_fs,max_years);


                %Step 1;
                [theta_hat_1 Q_hat]=fminunc(@(x)smd(x,WT,data_cell_smd,V_cell,sims_cell_final,0,theta_true,restrict_rho,restrict_sigma,known_fs,max_years),theta_hat_0_B,options_fmin);
                [theta_0 theta_hat_0_A theta_hat_0_B theta_hat_1]

                %Step 2: Overid weighted w/efficient weights;
                WT=get_weight(theta_hat_1,data_cell_smd,V_cell,sims_cell_WT,1,theta_true,restrict_rho,restrict_sigma,known_fs,max_years);
                [theta_hat_2 Q_hat]=fminunc(@(x)smd(x,WT,data_cell_smd,V_cell,sims_cell_final,1,theta_true,restrict_rho,restrict_sigma,known_fs,max_years),theta_hat_1,options_fmin);
                [theta_0 theta_hat_0_A theta_hat_0_B theta_hat_1 theta_hat_2]

                %Get SEs;
                [x,~,~,~,~,~,G_big] = lsqnonlin(@(x)get_m(x,data_cell_smd,V_cell,sims_cell_WT,1,theta_true,restrict_rho,restrict_sigma,known_fs,max_years),theta_hat_2,[],[],optimset('MaxIter',0,'Tolfun',100000));
                V_hat=inv(G_big'*WT*G_big)/J;
                SE=sqrt(diag(V_hat));


            %%%%%%% Display coefficient vector;
                [theta_0 theta_hat_1 theta_hat_2 SE];


            %%%%%%%%%%%%% Save results %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%;

                output={theta_hat_1,theta_hat_2,WT,G_big,V_hat,SE,Q_hat,sims_cell_final,sims_cell_WT};
                save(savename,'output');

        end;
   
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%%%%%% DISPLAY ESTIMATES %%%%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        
        if display==1;

                %Load estimates;
                load(savename);
                l=1;
                theta_hat_1=cell2mat(output(l)); l=l+1;
                theta_hat_2=cell2mat(output(l)); l=l+1;
                WT=cell2mat(output(l)); l=l+1;
                G_big=cell2mat(output(l)); l=l+1;
                V_hat=cell2mat(output(l)); l=l+1;
                SE=cell2mat(output(l)); l=l+1;
                Q_hat=cell2mat(output(l)); l=l+1;
                sims_cell_final=output(l); l=l+1;
                sims_cell_WT=output(l);
                
                %Coefficient vector;
                [theta_hat_1 theta_hat_2 SE]

                %Delta method for functions of params;
                results=get_output(theta_hat_2,K,restrict_rho,restrict_sigma,known_fs);
                [x,~,~,~,~,~,G] = lsqnonlin(@(x)get_output(x,K,restrict_rho,restrict_sigma,known_fs),theta_hat_2,[],[],optimset('MaxIter',0,'Tolfun',100000));
                covmat=G*V_hat*G';
                se_results=sqrt(diag(covmat));
                [results se_results];
                
                %J-test;
                moments=length(G_big(:,1));
                params=length(theta_hat_2);
                df=moments-params;
                [Q_hat (1-chi2cdf(Q_hat,df)) df]

                export_results =  {[results se_results] [Q_hat ;(1-chi2cdf(Q_hat,df)); df]};              

        end;         
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%%%%%% COMPUTE POSTERIORS %%%%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
        if posteriors==1;
            
                %%%%Load estimates;
                load(savename);
                l=1;
                theta_hat_1=cell2mat(output(l)); l=l+1;
                theta_hat_2=cell2mat(output(l)); l=l+1;
                WT=cell2mat(output(l)); l=l+1;
                G_big=cell2mat(output(l)); l=l+1;
                V_hat=cell2mat(output(l)); l=l+1;
                SE=cell2mat(output(l)); l=l+1;
                Q_hat=cell2mat(output(l)); l=l+1;
                sims_cell_final=output(l); l=l+1;
                sims_cell_WT=output(l);
                
                %%%Unpack coef vector;
                theta=theta_hat_2;
                sigma_beta_tilde=0;
                sigma_b_tilde=0;
                rho_tilde=0;
                chol_Sigma=zeros(4,4);
                theta_P=zeros(4,K);
                l=1;
                beta_Q=theta(l); l=l+1;
                if known_fs~=1;
                   theta_P=reshape(theta(l:(l+(4*K)-1)),[4 K]); l=l+(4*K);
                end;
                if known_fs==1;
                   theta_P(1:2,:)=reshape(theta(l:(l+(2*K)-1)),[2 K]); l=l+(2*K);
                end;
                for t=1:2;
                   chol_Sigma(t,t)=exp(theta(l)); l=l+1;
                 end;
                 if known_fs~=1;
                    for t=3:4;
                          chol_Sigma(t,t)=exp(theta(l)); l=l+1;
                    end;
                 end;
                 chol_Sigma(1,2)=theta(l); l=l+1;
                 if known_fs~=1;
                   chol_Sigma(1,3:end)=theta(l:(l+1)); l=l+2;
                   chol_Sigma(2,3:end)=theta(l:(l+1)); l=l+2;
                   chol_Sigma(3,4)=theta(l); l=l+1;
                   sigma_nu=exp(theta(l)); l=l+1;
                 end;
                 if restrict_sigma~=1;
                     sigma_beta_tilde=exp(theta(l)); l=l+1;
                     sigma_b_tilde=exp(theta(l)); l=l+1;
                 end;
                 if restrict_rho~=1;
                     rho_tilde=(exp(2*theta(l))-1)/(exp(2*theta(l))+1); l=l+1;
                 end;
                
                %%%%Format variance;
                inv_V=0*chol_V;
                for t=1:T;
                    V_temp=chol_V(:,:,t)'*chol_V(:,:,t);
                    inv_V(:,:,t)=inv(V_temp);
                end;
                
                %%%%Set parameters;
                post_params={known_fs,pooled,restrict_rho,restrict_sigma,max_years,Q,A,start_fixed};
                
                %%%Set starting vals;
                 mu=P*(theta_P');
                 mu(:,1)=mu(:,1)+beta_Q*Q;
                 mu_0=mu;
                 nu_0=zeros(J*sum(Q),1);
                 mu_wide=mu_0';
                 Theta_0=[mu_wide(:)];
                 if known_fs~=1;
                    Theta_0=[Theta_0; nu_0];
                 end;
                 Theta_init=Theta_0;
                 
                
                %%%%Compute posteriors;
                [alpha_post beta_post b_post Theta]=get_posteriors(alpha_hat,rho_hat,pi_hat,chol_V,P,theta_hat_2,post_params,Theta_init);
                if known_fs~=1;
                    theta=reshape(Theta(1:(4*J)),[4 J])';
                    beta=theta(:,1);
                    b=theta(:,2);
                    delta=theta(:,3);
                    chi=theta(:,4);
                    nu_long=Theta((4*J+1):end);
                end;
                if known_fs==1;
                    beta=zeros(J,1);
                    b=zeros(J,1);
                    delta=zeros(J,1);
                    chi=zeros(J,1);
                    nu_long=zeros(J*sum(Q),1);
                end
                
                %%%%Save posteriors;
                beta_post=beta_post-mean(beta_post);
                b_post=b_post-mean(b_post);
                beta=beta-mean(beta);
                b=b-mean(b);
                output_post={alpha_post,beta_post,b_post,beta,b,delta,chi,nu_long,P,chol_V,theta_hat_2,alpha_hat,rho_hat,pi_hat,Q,A};
                savename_post=['post_' savename];
                save(savename_post,'output_post');
                V=chol_V'*chol_V;
                J=length(alpha_hat);
                SE_alpha=sqrt(diag(V(1:J,1:J)));
                posteriors=[alpha_hat SE_alpha alpha_post beta_post b_post beta b P];
                dlmwrite(['posteriors_' savename '.csv'],posteriors);
                
        end;
         
