function [alpha_star beta_star b_star posteriors]=get_posteriors(alpha_hat,rho_hat,pi_hat,chol_V,P,theta,params,Theta_init);

    %%%%Options;
    options_fmin=struct('Algorithm','quasi-newton','Display','iter','MaxFunEvals',10000000,'MaxIter',10000,'GradObj','on','LargeScale','off','TolFun',1e-6);

    %%%Unpack parameters;
    l=1;
    known_fs=cell2mat(params(l)); l=l+1;
    pooled=cell2mat(params(l)); l=l+1;
    restrict_rho=cell2mat(params(l)); l=l+1;
    restrict_sigma=cell2mat(params(l)); l=l+1;
    max_years=cell2mat(params(l)); l=l+1;
    Q=cell2mat(params(l)); l=l+1;
    A=cell2mat(params(l)); l=l+1;
    start_fixed=cell2mat(params(l));
    K=length(P(1,:));
    J=length(alpha_hat(:,1));
    
    %%%Unpack coef. vector;
    sigma_beta_tilde=0;
    sigma_b_tilde=0;
    rho_tilde=0;
    chol_Sigma=zeros(4,4);
    theta_P=zeros(4,K);
    l=1;
    beta_Q=theta(l); l=l+1;
    if known_fs~=1;
       theta_P=reshape(theta(l:(l+(4*K)-1)),[4 K]); l=l+(4*K);
    end;
    if known_fs==1;
       theta_P(1:2,:)=reshape(theta(l:(l+(2*K)-1)),[2 K]); l=l+(2*K);
    end;
    for t=1:2;
       chol_Sigma(t,t)=exp(theta(l)); l=l+1;
     end;
     if known_fs~=1;
        for t=3:4;
              chol_Sigma(t,t)=exp(theta(l)); l=l+1;
        end;
     end;
     chol_Sigma(1,2)=theta(l); l=l+1;
     if known_fs~=1;
       chol_Sigma(1,3:end)=theta(l:(l+1)); l=l+2;
       chol_Sigma(2,3:end)=theta(l:(l+1)); l=l+2;
       chol_Sigma(3,4)=theta(l); l=l+1;
       sigma_nu=exp(theta(l)); l=l+1;
     end;
     if restrict_sigma~=1;
         sigma_beta_tilde=exp(theta(l)); l=l+1;
         sigma_b_tilde=exp(theta(l)); l=l+1;
     end;
     if restrict_rho~=1;
         rho_tilde=(exp(2*theta(l))-1)/(exp(2*theta(l))+1); l=l+1;
     end;
     
     %Format data;
        V=chol_V'*chol_V;
     
     %%%Naive posteriors;
     
        %Variance of OLS;
        V_e=diag(V(1:J,1:J));
        mu_alpha=P*inv(P'*P)*P'*alpha_hat;
        res_alpha=alpha_hat - mu_alpha;
        V_alpha=var(res_alpha)-mean(V_e);
        lambda=V_alpha./(V_alpha+V_e);
        alpha_star=lambda.*alpha_hat+(1-lambda).*mu_alpha;
        
     
     %%%Posteriors treating FS as known;

         %Data;
         L=sum(Q);
         rho_short=rho_hat(1:L);
         Pi=pi_hat(:,1:L)';
         V_temp=V(1:(L+J),1:(L+J));
         inv_V=inv(V_temp);
         
         %Prior means;
         mu=P*(theta_P');
         mu(:,1)=mu(:,1)+beta_Q*Q;
         mu_0=mu;
         mu_wide=mu_0';
         L=sum(Q(:,1));
         nu_0=zeros(J,L);
         Theta_0=[mu_wide(:)];
         if known_fs~=1;
              Theta_0=[Theta_0; 0*nu_0(:)];
         end;
         beta_0=mu(:,1);
         b_0=mu(:,2);

         %Prior variance;
         Sigma=chol_Sigma'*chol_Sigma;
         Sigma_temp=Sigma(1:2,1:2);
         Sigma_big=kron(Sigma_temp,eye(J));
         inv_Sigma=inv(Sigma_big);
         v_beta_beta=inv_Sigma(1:J,1:J);
         v_beta_b=inv_Sigma(1:J,(J+1):end);
         v_b_b=inv_Sigma((J+1):end,(J+1):end);
         
         %Variance of measurement error;
         v_alpha_alpha=inv_V(1:J,1:J);
         v_alpha_rho=inv_V(1:J,(J+1):end);
         v_rho_rho=inv_V((J+1):end,(J+1):end);


         %Posterior variance;
         vstar_beta_beta=v_alpha_alpha+Pi'*v_alpha_rho'+v_alpha_rho*Pi+Pi'*v_rho_rho*Pi+v_beta_beta;
         vstar_beta_b=v_alpha_alpha+Pi'*v_alpha_rho'+v_beta_b;
         vstar_b_b=v_alpha_alpha+v_b_b;

         
         %Weights;
         B=vstar_b_b*inv(vstar_beta_b)*vstar_beta_beta - (v_alpha_alpha+v_alpha_rho*Pi+v_beta_b');
         W1=inv(B)*(vstar_b_b*inv(vstar_beta_b)*(v_alpha_alpha+Pi'*v_alpha_rho') - v_alpha_alpha);
         W2=inv(B)*(vstar_b_b*inv(vstar_beta_b)*(v_alpha_rho+Pi'*v_rho_rho)-v_alpha_rho);
         
         %Posterior means;
         beta_star=W1*(alpha_hat - b_0) + W2*rho_short + (eye(J) - W1 - W2*Pi)*beta_0;
         b_star=inv(vstar_b_b)*(v_alpha_alpha*alpha_hat+v_alpha_rho*rho_short+v_beta_b*beta_0+v_b_b*b_0-vstar_beta_b'*beta_star);

     %%%Posteriors with random first stage;
         posteriors=[];
         if known_fs~=1;
             
             %%%%Setup;
             inv_V=inv(V);
             Omega_hat=[alpha_hat; rho_hat; pi_hat(:)];
             Sigma_0=zeros(4*J+J*L,4*J+J*L);
             Sigma_0(1:(4*J),1:(4*J))=kron(eye(J),Sigma);
             Sigma_0((4*J+1):end,(4*J+1):end)=sigma_nu^2*eye(J*L);
             inv_Sigma_0=inv(Sigma_0);   
             parameters={J,L,1,Q,A,max_years};
             
             %Starting vals for beta and b equal to fixed FS ests;
                if start_fixed==1;
                    theta=reshape(Theta_init(1:(4*J)),[4 J])';
                    nu_long=Theta_init((4*J+1):end);
                    theta(:,1)=beta_star;
                    theta(:,2)=b_star;
                    theta_wide=theta';
                    Theta_init=[theta_wide(:); nu_long];
                end;
             
             %%%%Maximize;
            [posteriors]=fminunc(@(x)posterior_density(x,Omega_hat,Theta_0,inv_V,inv_Sigma_0,parameters),Theta_init,options_fmin);

             
         end;
     
     
     
     
     
     