

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% This program runs the policy %%%%
%%% simulations %%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%% SETUP %%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
        %%%Set iteration number;
        iteration=1;
        num_runs=500;
        
        for it=iteration:(iteration+num_runs-1);
            
            rng(it);
            disp('Iteration:')
            it
            
        
        %Model;
        model='3';
        grade=6;
        sfx=0;
        known_fs=0;
        pooled=0;
        years_data=8;
        restrict_rho=1;
        restrict_sigma=1;

        %Optimization options;
            estimate_model=0;
            start_fixed=0;
            options_fmin=struct('Display','iter','MaxFunEvals',100000,'MaxIter',500,'GradObj','off','TolFun',1e-4);
            R_WT=1000;
            R_final=500;
            
        %Max exposure;
        max_years=1;
        if grade==7;
            J=46;
            max_years=2;
        end;
        if grade==8;
            J=43;
            max_years=3;
        end;
        
        %Format model name;
            savename=['results_' model];
            if sfx==1;
                savename=[savename '_sfx'];
            end;
            if pooled==1;
                savename=[savename '_pooled'];
            end;
            if known_fs==1;
                savename=[savename '_known_fs'];
            end;
            savename_final=savename;
            if years_data~=8;
                savename_final=[savename '_' num2str(years_data) 'years'];
            end;

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%% Unpack data %%%%%%%%%%%%%%%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        
    
        %%%Data;
            savename_post=['post_' savename];
            load(savename_post);
            l=9;
            P=cell2mat(output_post(l)); l=l+1;
            chol_V=cell2mat(output_post(l)); l=l+1;
            theta_hat_2=cell2mat(output_post(l)); l=l+3;
            pi_true=cell2mat(output_post(l)); l=l+1;
            Q=cell2mat(output_post(l)); l=l+1;
            A=cell2mat(output_post(l));
            J=length(Q(:,1));
            T=length(Q(1,:));
            K=length(P(1,:));
            any_Q=1-prod(1-Q,2);
   
        %%%Parameters;
            theta=theta_hat_2;
            sigma_beta_tilde=0;
            sigma_b_tilde=0;
            rho_tilde=0;
            sigma_nu=0;
            chol_Sigma=zeros(4,4);
            theta_P=zeros(4,K);
            l=1;
            beta_Q=theta(l); l=l+1;
            if known_fs~=1;
                theta_P=reshape(theta(l:(l+(4*K)-1)),[4 K]); l=l+(4*K);
            end;
            if known_fs==1;
                theta_P(1:2,:)=reshape(theta(l:(l+(2*K)-1)),[2 K]); l=l+(2*K);
            end;
            for t=1:2;
                chol_Sigma(t,t)=exp(theta(l)); l=l+1;
            end;
            if known_fs~=1;
                for t=3:4;
                    chol_Sigma(t,t)=exp(theta(l)); l=l+1;
                end;
            end;
            chol_Sigma(1,2)=theta(l); l=l+1;
            if known_fs~=1;
                chol_Sigma(1,3:end)=theta(l:(l+1)); l=l+2;
                chol_Sigma(2,3:end)=theta(l:(l+1)); l=l+2;
                chol_Sigma(3,4)=theta(l); l=l+1;
                sigma_nu=exp(theta(l)); l=l+1;
            end;
            if restrict_sigma~=1;
                sigma_beta_tilde=exp(theta(l)); l=l+1;
                sigma_b_tilde=exp(theta(l)); l=l+1;
            end;
            if restrict_rho~=1;
                rho_tilde=(exp(2*theta(l))-1)/(exp(2*theta(l))+1); l=l+1;
            end;
            Sigma=chol_Sigma'*chol_Sigma;
            Sigma_tilde=[sigma_beta_tilde^2 rho_tilde*sigma_beta_tilde*sigma_b_tilde; ...
                         rho_tilde*sigma_beta_tilde*sigma_b_tilde sigma_b_tilde^2];
    
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%% Simulate data %%%%%%%%%%%%%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
             %%Random coefficients;  
                beta=zeros(J,T);
                b=zeros(J,T);
                delta=zeros(J,T);
                chi=zeros(J,T);
                beta_tilde=zeros(J,T);
                b_tilde=zeros(J,T);
                mu=P*(theta_P');
                mu(:,1)=mu(:,1)+beta_Q*any_Q;
                omega=mu+mvnrnd(zeros(1,4),Sigma,J);
                for t=1:T;
                    if restrict_sigma~=1;
                        omega_temp=mvnrnd([0 0],Sigma_tilde,J);
                        beta_tilde(:,t)=omega_temp(:,1);
                        b_tilde(:,t)=omega_temp(:,2);
                    end;
                    beta(:,t)=omega(:,1)+beta_tilde(:,t);
                    b(:,t)=omega(:,2)+b_tilde(:,t);
                    delta(:,t)=omega(:,3);
                    chi(:,t)=omega(:,4);
                end;
                alpha=beta+b;

                %%First stage;
                pi=pi_true;
                nu=sigma_nu*randn(J,J,T);
                chi_tilde=repmat(reshape(chi,[J 1 T]),[1 J 1])+nu;
                pi_jj=exp(delta)./(1+exp(delta));
                rho=zeros(J,T);
                for t=1:T;
                    if known_fs~=1;
                        A_temp=A(:,t);
                        for j=1:J;
                            u_temp=exp(chi_tilde(:,j,t)).*A_temp;
                            u_temp(j)=0;
                            fallback=u_temp./(1+sum(u_temp));
                            pi(:,j,t)=-pi_jj(j,t).*fallback;
                            pi(j,j,t)=pi_jj(j,t);
                            pi(:,j,t)=pi(:,j,t)*Q(j,t);
                            pi(:,j,t)=pi(:,j,t)*max_years;
                        end;
                    end;
                    rho(:,t)=Q(:,t).*(sum(pi(:,:,t).*repmat(beta(:,t),[1 J]),1)');
                end;

                %%Measurement error process;
                chol_V=chol_V*sqrt(8/years_data);
                V=zeros(J*(2+J),J*(2+J),T);
                V_alpha=zeros(J,T);
                V_rho=zeros(J,T);
                e=zeros(J*(2+J),T);
                for t=1:T;
                    V(:,:,t)=(chol_V(:,:,t)'*chol_V(:,:,t));
                    e(:,t)=mvnrnd(zeros(1,J*(2+J)),V(:,:,t));
                    V_alpha(:,t)=diag(V(1:J,1:J,t));
                    V_rho(:,t)=diag(V((J+1):(2*J),(J+1):(2*J)));
                end;
                e_pi_long=e(((2*J)+1):end,:);
                e_pi=zeros(J,J,T);
                if known_fs~=1;
                    for t=1:T;
                        e_temp=e_pi_long(:,t);
                        e_pi(:,:,t)=reshape(e_temp,[J J]);
                    end;
                end;

                %%Observed data vector;
                alpha_hat=(alpha+e(1:J,:)).*A;
                rho_hat=(rho+e((J+1):(2*J),:)).*Q;
                pi_hat=pi+e_pi;
                for t=1:T;
                    pi_hat(:,:,t)=pi_hat(:,:,t).*repmat(A(:,t),[1 J]).*repmat(Q(:,t)',[J 1]);
                end;

                %%Cell arrays to pass to optimizers;
                data_cell_smd={alpha_hat,rho_hat,pi_hat,P,A,Q};
                V_cell={V_alpha,V_rho};
                
            
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%% Fit random coefs model %%%%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        
        if estimate_model==1;
                %Simulation draws;

                        %Weighting matrix;
                        sims_omega_j=randn(J,4,R_WT);
                        sims_omega_tilde=randn(J,2,T,R_WT);
                        sims_nu=randn(J,J,T,R_WT);
                        sims_e=randn(J*(2+J),T,R_WT);
                        e_sim=zeros(J*(2+J),T,R_WT);
                        for t=1:T;
                            for r=1:R_WT;
                                e_sim(:,t,r)=((sims_e(:,t,r)')*chol_V(:,:,t))';
                            end;
                        end;
                        sims_cell_WT={sims_omega_j,sims_omega_tilde,sims_nu,e_sim};

                        %Final sims;
                        sims_omega_j=randn(J,4,R_final);
                        sims_omega_tilde=randn(J,2,T,R_final);
                        sims_nu=randn(J,J,T,R_final);
                        sims_e=randn(J*(2+J),T,R_final);
                        e_sim=zeros(J*(2+J),T,R_final);
                        for t=1:T;
                            for r=1:R_final;
                                e_sim(:,t,r)=((sims_e(:,t,r)')*chol_V(:,:,t))';
                            end;
                        end;
                        sims_cell_final={sims_omega_j,sims_omega_tilde,sims_nu,e_sim};

                %Step 1;
                theta_0=theta_hat_2;
                WT=get_weight(theta_0,data_cell_smd,V_cell,sims_cell_WT,1,theta_0,restrict_rho,restrict_sigma,known_fs,max_years);
                WT=eye(size(WT));
                [theta_hat_1 Q_hat]=fminunc(@(x)smd(x,WT,data_cell_smd,V_cell,sims_cell_final,0,theta_0,restrict_rho,restrict_sigma,known_fs,max_years),theta_0,options_fmin);

                %Step 2: Overid weighted w/efficient weights;
                WT=get_weight(theta_hat_1,data_cell_smd,V_cell,sims_cell_WT,1,theta_0,restrict_rho,restrict_sigma,known_fs,max_years);
                [theta_hat_2 Q_hat]=fminunc(@(x)smd(x,WT,data_cell_smd,V_cell,sims_cell_final,1,theta_0,restrict_rho,restrict_sigma,known_fs,max_years),theta_hat_1,options_fmin);
        end;

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%% Get posteriors %%%%%%%%%%%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
                %%%%Format variance;
                inv_V=0*chol_V;
                for t=1:T;
                    V_temp=chol_V(:,:,t)'*chol_V(:,:,t);
                    inv_V(:,:,t)=inv(V_temp);
                end;
                
                %%%%Set parameters;
                post_params={known_fs,pooled,restrict_rho,restrict_sigma,max_years,Q,A,start_fixed};
                
                %%%Set mean;
                 mu=P*(theta_P');
                 mu(:,1)=mu(:,1)+beta_Q*Q;
                 mu_0=mu;
                 mu_wide=mu_0';
                 L=sum(Q(:,1));
                 nu_0=nu(:,1:L);
                 Theta_0=[mu_wide(:)];
                 if known_fs~=1;
                    Theta_0=[Theta_0; 0*nu_0(:)];
                 end;

                %%%Set starting values;
                Theta_init_wide=[beta b delta chi]';
                Theta_init=[Theta_init_wide(:); nu_0(:)];
                
                %%%%Compute posteriors;

                [alpha_post beta_post_fixed b_post_fixed Theta]=get_posteriors(alpha_hat,rho_hat,pi_hat,chol_V,P,theta_hat_2,post_params,Theta_init);
                beta_post=zeros(J,1);
                b_post=zeros(J,1);
                if known_fs~=1;
                    theta=reshape(Theta(1:(4*J)),[4 J])';
                    beta_post=theta(:,1);
                    b_post=theta(:,2);
                end;
                
                %%%Save posteriors;
                output=[beta alpha_hat sqrt(V_alpha) beta_post_fixed beta_post];
                save(['postsim_' savename_final num2str(it)],'output');
    
    
    
        end;
    
    
    
    
    
    
    