function [obj d_obj]=posterior_density(Theta,Omega_hat,Theta_0,inv_V,inv_Sigma_0,parameters);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%% SETUP %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    %%%Unpack;
    l=1;
    J=cell2mat(parameters(l)); l=l+1;
    L=cell2mat(parameters(l)); l=l+1;
    T=cell2mat(parameters(l)); l=l+1;
    Q=cell2mat(parameters(l)); l=l+1;
    A=cell2mat(parameters(l)); l=l+1;
    max_years=cell2mat(parameters(l)); l=l+1;
    
    %%%Format;
    theta=reshape(Theta(1:(4*J)),[4 J])';
    beta=theta(:,1);
    b=theta(:,2);
    delta=theta(:,3);
    chi=theta(:,4);
    nu_long=Theta((4*J+1):end);
    nu_long=[nu_long; zeros(J*J - length(nu_long),1)];
    nu=reshape(nu_long,[J J]);
    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%% OBJ FN %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    %%%Construct omega params;
            chi_tilde=repmat(reshape(chi,[J 1 T]),[1 J 1])+nu;
            pi_jj=max_years*(exp(delta)./(1+exp(delta)));
            pi=zeros(J,J,T);
            rho=zeros(J,T);
            for t=1:T;
                    A_temp=A(:,t);
                    for j=1:J;
                        u_temp=exp(chi_tilde(:,j,t)).*A_temp;
                        u_temp(j)=0;
                        fallback=u_temp./(1+sum(u_temp));
                        pi(:,j,t)=-pi_jj(j,t).*fallback;
                        pi(j,j,t)=pi_jj(j,t);
                        pi(:,j,t)=pi(:,j,t)*Q(j,t);
                    end;
                rho(:,t)=Q(:,t).*(sum(pi(:,:,t).*repmat(beta(:,t),[1 J]),1)');
            end;

            %Reformat random coefs;
            alpha=beta+b;
            rho=zeros(J,T);
            for t=1:T;
                rho(:,t)=Q(:,t).*(sum(pi(:,:,t).*repmat(beta(:,t),[1 J]),1)');
            end;
            
    %%%%%Format Omega;
        Omega=[alpha; rho; pi(:)];
    
    %%%%Likelihood;
        logL= -0.5*(Omega_hat - Omega)'*inv_V*(Omega_hat - Omega) - 0.5*(Theta - Theta_0)'*inv_Sigma_0*(Theta - Theta_0);
        obj=-logL;
        
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%% GRADIENT %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    %%%%%%%%%%%%%%Derivatives w.r.t beta;
    
        %%%%dalpha/dbeta;
        dalpha_dbeta=eye(J);
        
        %%%%drho/dbeta;
        drho_dbeta=pi';
        
        %%%%%dpi/dbeta;
        dpi_dbeta=zeros(J*J,J);

    
    %%%%%%%%%%%%%%Derivatives w.r.t b;
    
        %%%%dalpha/db;
        dalpha_db=eye(J);
        
        %%%%drho/db;
        drho_db=zeros(J,J);
        
        %%%%%dpi/db;
        dpi_db=zeros(J*J,J); 

    %%%%%%%%%%%%%%Derivatives w.r.t delta;
    
        %%%%dalpha/ddelta;
        dalpha_ddelta=zeros(J,J);
        
        %%%%drho/ddelta;
        drho_ddelta=diag((1-(pi_jj/max_years)).*rho);
        
        %%%%%dpi/ddelta;
        dpi_ddelta=zeros(J*J,J);
        for l=1:J;
            dpi_ddelta(:,l)=Q(l)*(1-(pi_jj(l)/max_years))*[zeros((l-1)*J,1); pi(:,l); zeros(J^2 - l*J,1)];
        end;
        
    %%%%%%%%%%%%%%Derivatives w.r.t chi and nu;
    
        %%%%dalpha/dchi;
        dalpha_dchi=zeros(J,J);
        dalpha_dnu=zeros(J,J*sum(Q));
        
        %%%%drho/dchi and dpi/dchi;
        drho_dchi=zeros(J,J);
        dpi_dchi=zeros(J*J,J);
        drho_dnu=zeros(J,J*sum(Q));
        dpi_dnu=zeros(J*J,J*sum(Q));
        for l=1:J;

            %Calculate derivs for all pi's in lotto l;
            derivs_temp=Q(l)*(repmat(pi(:,l),[1 J]).*repmat(pi(:,l)',[J 1])/((1-Q(l))+pi_jj(l)));
            diags=Q(l)*(pi(:,l).*(1 + pi(:,l)/((1-Q(l))+pi_jj(l))));
            derivs_temp = derivs_temp - diag(diag(derivs_temp)) + diag(diags);
            derivs_temp(l,:)=0;
            derivs_temp(:,l)=0;

            %Fill in RF and FS derivs;
            drho_dchi(l,:)=sum(derivs_temp.*repmat(beta,[1 J]),1);
            dpi_dchi((1+(l-1)*J):(l*J),:)=derivs_temp;

            %Fill in nu dervis;
            if l<=sum(Q);
                drho_dnu(l,(1+J*(l-1)):(J*l))=drho_dchi(l,:);
                dpi_dnu((1+(l-1)*J):(l*J),(1+(l-1)*J):(l*J))=dpi_dchi((1+(l-1)*J):(l*J),:);
            end;

        end;

    %%%%%%%Form derivative matrix;
    
        dOmega_dTheta=[dalpha_dbeta dalpha_db dalpha_ddelta dalpha_dchi dalpha_dnu; ...
                       drho_dbeta drho_db drho_ddelta drho_dchi drho_dnu; ...
                       dpi_dbeta dpi_db dpi_ddelta dpi_dchi dpi_dnu];
    
    
    %%%%%%%Form final gradient;
        d1=((Omega - Omega_hat)'*inv_V*dOmega_dTheta)';
        d_temp=reshape(d1(1:(4*J)),[J 4])';
        d1=[d_temp(:); d1(((4*J)+1):end)];
        d2=((Theta - Theta_0)'*inv_Sigma_0)';
        d_obj=d1+d2;
    
        
    
        
        
        
        
        
             