%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Chris Walters %%%%%%%%%%%%%%%%%%%%
%%%%% This program simulates %%%%%%%%%%
%%%%% VAM-based policies %%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%%%%% SET UP MATLAB %%%%%%%%%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
        %Clear;
        clear all;
        
        %Model specifications;
        model='3';
        grade=6;
        sfx=1;
        known_fs=0;
        pooled=1;
        years_data=8;
        restrict_sigma=1;
        restrict_rho=1;
        
        %Simulations;
        min_sim=1;
        max_sim=500;
        
        %Parameters of DGP;
        T=1+7*(1-pooled);
        J=50;
        max_years=1;
        if grade==7;
            J=46;
            max_years=2;
        end;
        if grade==8;
            J=43;
            max_years=3;
        end;
        
        %Format model name;
            savename=['results_' model];
            if sfx==1;
                savename=[savename '_sfx'];
            end;
            if pooled==1;
                savename=[savename '_pooled'];
            end;
            if known_fs==1;
                savename=[savename '_known_fs'];
            end;
            savename_final=savename;
            if years_data~=8;
                savename_final=[savename '_' num2str(years_data) 'years'];
            end;
            
        %Load covs;
               sects=dlmread('VAM_sects.csv');
               if pooled~=1;
                   sects=dlmread('VAM_sects_long.csv');
               end;   
               if grade==7;
                   sects=dlmread('VAM_sectsg7.csv');
               end;
               if grade==8;
                   sects=dlmread('VAM_sectsg8.csv');
               end;
               charter=sects(:,1);
               pilot=sects(:,2);
               P=[charter pilot ones(J,1)];
               if sfx==0;
                   P=ones(J,1);
               end;
               K=length(P(1,:));
        
        
        %Unpack parameter vector;
            load(savename);
            theta=cell2mat(output(2));
            sigma_beta_tilde=0;
            sigma_b_tilde=0;
            rho_tilde=0;
            chol_Sigma=zeros(4,4);
            theta_P=zeros(4,K);
            l=1;
            beta_Q=theta(l); l=l+1;
            if known_fs~=1;
                theta_P=reshape(theta(l:(l+(4*K)-1)),[4 K]); l=l+(4*K);
            end;
            if known_fs==1;
                theta_P(1:2,:)=reshape(theta(l:(l+(2*K)-1)),[2 K]); l=l+(2*K);
            end;
            for t=1:2;
                chol_Sigma(t,t)=exp(theta(l)); l=l+1;
            end;
            if known_fs~=1;
                for t=3:4;
                    chol_Sigma(t,t)=exp(theta(l)); l=l+1;
                end;
            end;
            chol_Sigma(1,2)=theta(l); l=l+1;
            if known_fs~=1;
                chol_Sigma(1,3:end)=theta(l:(l+1)); l=l+2;
                chol_Sigma(2,3:end)=theta(l:(l+1)); l=l+2;
                chol_Sigma(3,4)=theta(l); l=l+1;
                sigma_nu=exp(theta(l)); l=l+1;
            end;
            if restrict_sigma~=1;
                sigma_beta_tilde=exp(theta(l)); l=l+1;
                sigma_b_tilde=exp(theta(l)); l=l+1;
            end;
            if restrict_rho~=1;
                rho_tilde=(exp(2*theta(l))-1)/(exp(2*theta(l))+1); l=l+1;
            end;
        
        
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%%%%% ANALYZE SIM RESULTS %%%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
        %Results storage;
        mse=[];
        gains=[];
        misclassify=[];
    
        %Loop over sims;
        for s=min_sim:max_sim;
            
            %Results storage;
            temp_results=[];
            
            %See if this sim exists;
            
            try;
                %Model name;
                savename=savename_final;
                loadname=['postsim_' savename num2str(s)];
                

                %Load results;
                load(loadname);

                %Unpack results;
                l=1;
                beta=output(:,l); l=l+1;
                alpha_hat=output(:,l); l=l+1;
                SE_alpha_hat=output(:,l); l=l+1;
                beta_post_fixed=output(:,l); l=l+1;
                beta_post=output(:,l);

                %Get conventional posteriors;
                mu_alpha=P*inv(P'*P)*P'*alpha_hat;
                res_alpha=alpha_hat - mu_alpha;
                V_alpha=var(res_alpha)-mean(SE_alpha_hat.^2);
                lambda=V_alpha./(V_alpha+(SE_alpha_hat.^2));
                alpha_post=lambda.*alpha_hat+(1-lambda).*mu_alpha;


                %Normalize everthing relative to mean included school;
                beta=beta-mean(beta);
                beta_post_fixed=beta_post_fixed-mean(beta_post_fixed);
                beta_post=beta_post - mean(beta_post);
                alpha_post=alpha_post-mean(alpha_post);
                
                %Get MSE;            
                mse_beta_post=mean((beta_post - beta).^2);
                mse_alpha_post=mean((alpha_post - beta).^2);
                mse_beta_post_fixed=mean((beta_post_fixed - beta).^2);
                mse_results=[mse_alpha_post mse_beta_post_fixed];
                if known_fs~=1;
                    mse_results=[mse_results mse_beta_post];
                end;
                
                %%%%%Policy sims;
                
                    %%%%If 7th or 8th grade, cumulative years of exposure;
                    if grade==7;
                        beta=beta;
                    end;
                    if grade==8;
                        beta=beta;
                    end;
                
                    %%%Save avg charter va and restrict to district
                    %%%schools;
                    charter_va=mean(beta(charter==1));
                    beta=beta(charter~=1);
                    beta_post_fixed=beta_post_fixed(charter~=1);
                    alpha_post=alpha_post(charter~=1);
                    beta_post=beta_post(charter~=1);
                   
                    
                    %%%Array of value-added estimates to use;
                    beta_va_list=[beta alpha_post];
                    if known_fs~=1;
                        beta_va_list=[beta_va_list beta_post];
                    end;
                
                    %%%%Loop over va estimates for policy sims;
                    gains_results=[];
                    misclassify_results=[];
                    for b=1:length(beta_va_list(1,:));
                        
                        %Value-added measure;
                        beta_va=beta_va_list(:,b);
                        
                        %%%%Gains;
                        
                            %Average school;
                            [minvalue minindex]=min(beta_va,[],1);
                            min_va=beta(minindex);
                            diff=mean(beta) - min_va;
                            gains_results=[gains_results diff];

                            %Average above_median school;
                            p_high=prctile(beta_va,50);
                            high=(beta_va>=p_high);
                            diff=mean(beta(high==1)) - min_va;
                            gains_results=[gains_results diff];

                            %Average top-quintile school;
                            p_high=prctile(beta_va,80);
                            high=(beta_va>=p_high);
                            diff=mean(beta(high==1)) - min_va;
                            gains_results=[gains_results diff];

                            %Average charter school;
                            diff=charter_va - min_va;
                            gains_results=[gains_results diff];
                            
                        %%%%Misclassification;
                        
                            %Lowest decile, quintile, tercile;
                            ranklist=[10 20 33.3];
                            for t=1:length(ranklist);
                                p=ranklist(t);
                                p_low=prctile(beta,p);
                                low=(beta<=p_low);
                                p_low_post=prctile(beta_va,p);
                                low_post=(beta_va<=p_low_post);
                                not_low_post=1-low_post;
                                rate=mean(not_low_post(low==1));
                                misclassify_results=[misclassify_results rate];
                            end;
                            
                            %Highest decile, quintile, tercile;
                            ranklist=[90 80 66.6];
                            for t=1:length(ranklist);
                                p=ranklist(t);
                                p_high=prctile(beta,p);
                                high=(beta>=p_high);
                                p_high_post=prctile(beta_va,p);
                                high_post=(beta_va>=p_high_post);
                                not_high_post=1-high_post;
                                rate=mean(not_high_post(high==1));
                                misclassify_results=[misclassify_results rate];
                            end;

                            %Rank coefficient;
                            rank_true=tiedrank(beta);
                            rankmat_post=[ones(length(beta),1) tiedrank(beta_va)];
                            rank_coef=inv(rankmat_post'*rankmat_post)*rankmat_post'*rank_true;
                            misclassify_results=[misclassify_results rank_coef(2)];

                        
                    end;
                        
              
                %Update output;
                mse=[mse; mse_results];
                gains=[gains; gains_results];
                misclassify=[misclassify; misclassify_results];
                
            end;
  
            
        end;
        
        %Count sims;
        length(mse(:,1))
        
        %MSE
        sqrt(mean(mse))
        
        %Error rates;
        [mean(gains); ...
        std(gains)]
    
        %Misclaissification rates;
        [mean(misclassify); ...
         std(misclassify)]
        
        
        