/******************************************************************/
/*     Byyear program for "Leveraging Lotteries for Value-Added   */
/*     Generates "first step" OLS, RF, and FS estimates by year   */
/*            Originally created February 10 2016 by PDH          */
/*                 Last updated April 4 2017 by PDH               */
/******************************************************************/

clear all
cap log close
set more off
set trace off
set matsize 11000
set maxvar 32767
set segmentsize 512m
set seed 42
adopath + "Z:\ado\plus\"

global asgndata   "Z:/BPS/BPS assignment data/"
global cleandata  "Z:/BPS/Peter/BPSVA/Data/"
global distdata   "Z:/BPS/Peter/BPSVA/Data/location/distance files/"
global chartdata  "Z:/charters/MA Lotteries/all lotto data/school by year files/"
global examdata   "Z:/BPS/Exam Schools/Data/"

global programs   "Z:/BPS/Peter/BPSVA/Programs/"
global tables     "Z:/BPS/Peter/BPSVA/Tables/"
global graphs     "Z:/BPS/Peter/BPSVA/Graphs/"

/* timestamps */
local simsdate    "2015 01 19"
local charterdate "2014 08 10"
local rundate     "05032016"

/* all baseline controls */
local all_covs    "bl_female bl_sch_female bl_black bl_sch_black bl_hisp bl_sch_hisp bl_white bl_sch_white bl_asian bl_sch_asian bl_sped bl_sch_sped bl_lep bl_sch_lep bl_foodfred bl_sch_foodfred bl_susp bl_sch_susp bl_truancy bl_sch_truancy bl_m bl_sch_m bl_e bl_sch_e y_*"

/* baseline VAM controls */
local VAM_covs_m1 "y_*"
local VAM_covs_m2 "bl_female bl_black bl_hisp bl_white bl_asian bl_sped bl_lep bl_foodfred bl_susp bl_truancy y_*"
local VAM_covs_m3 "bl_female bl_black bl_hisp bl_white bl_asian bl_sped bl_lep bl_foodfred bl_susp bl_truancy bl_y_2013 bl_m bl_m2 bl_m3 bl_e bl_e2 bl_e3 y_*"

/* baseline lottery controls */
local QE_covs_t0 "one"
local QE_covs_t1 "y_* bl_m bl_m2 bl_m3 bl_e bl_e2 bl_e3"
local QE_covs_t2 "y_* bl_m bl_m2 bl_m3 bl_e bl_e2 bl_e3 bl_female bl_black bl_hisp bl_white bl_asian bl_sped bl_lep bl_foodfred bl_susp bl_truancy bl_y_2013"

/* school sectors */
local charters "RoxP 4100205 4190305 MATCH 4280305 APR 4390050 4430205 4440205 UP 4490305 4570205 4590305 BosP 4630205 4750505 4810550 4900305"
local pilots   "1459 2360 2440 4160 4285 4331 4600 4661 4680 350077 350565"
local early_entry "350077 350080 4100205 4280305 4430205 4440205 4490305 4570205 4590305 4630205 4750505 4810550 4900305"

/* program toggles  */
local fstep_ests   = 0
local encode_ests  = 0

/* sampling options */
local min_qesample = 50 
local ref_school   = 350691                            

/* model options */
local VAM_t        = "m"  
local fdiff        = "d" 

/*****************************************/
/* JOINT ESTIMATION OF FIRST STEP PARAMS */
/*****************************************/
if `fstep_ests' == 1 {

	forval y = 2007/2014 {
	/* Loop over VAM models */
		forval model =2/2 {
			/* Loop over QE tests */
			forval test = 2/2 {

			use "${cleandata}ms_obs_6th", clear
			gen masscode = masscode6
			merge m:1 masscode using "${cleandata}ms_fc_sc_schools_v2", keep(master match)
			tostring masscode, replace
			replace masscode = sch if _merge==3
			drop _merge sch
			
			keep if year6 == `y'
			if `y' == 2014 {
				local VAM_covs_m1 ""
				local VAM_covs_m2 "bl_female bl_black bl_hisp bl_white bl_asian bl_sped bl_lep bl_foodfred"
				local VAM_covs_m3 "bl_female bl_black bl_hisp bl_white bl_asian bl_sped bl_lep bl_foodfred bl_m bl_m2 bl_m3 bl_e bl_e2 bl_e3"
				local QE_covs_t2 "bl_m bl_m2 bl_m3 bl_e bl_e2 bl_e3 bl_female bl_black bl_hisp bl_white bl_asian bl_sped bl_lep bl_foodfred"
			}
			local QE_covs `QE_covs_t`test''
			
			local VAM_schs ""
			local N_VAM_schs = 0
			qui levelsof masscode, local(schools)
			foreach s in `schools' {
				if "`s'" != "`ref_school'" {
					gen D_`s' = (masscode == "`s'")
					local VAM_schs "`VAM_schs' `s'"
					local ++N_VAM_schs
				}
			}

			preserve
			clear
			set obs `=`N_VAM_schs'+1'
			gen sch = "`ref_school'" if _n==1
			gen sch_num = 0 if _n==1
			forval j = 1/`N_VAM_schs' {
				replace sch = word("`VAM_schs'",`j') if _n==`j'+1
				replace sch_num = `j' if _n==`j'+1
			}
			save "${cleandata}VAM_sch_xwalk_year`y'", replace
			restore

			merge 1:1 sasid using "${cleandata}ms_prefs_6th", keep(master match)

			foreach var of varlist apply_* {					
				replace `var' = 0 if `var' == .
			}
			drop _merge
		

			preserve
			gen samp = "VAM"
			if "`fdiff'" == "" {
				gen Y = c_bos_`VAM_t'rawsc6
			}
			if "`fdiff'" == "d" {
				gen Y = c_bos_`VAM_t'rawsc6 - bl_`VAM_t'				
			}
			keep sasid samp Y D_* `VAM_covs_m`model''
			tempfile samp_VAM
			save `samp_VAM', replace
			restore
			
			merge 1:m sasid using "${cleandata}ms_fc_sc_exper_6th_v2", keep(master match)
			drop _merge
			
			drop ch1 ch2 fc_hurdle sc_hurdle
			foreach var of varlist riskset_* qual_* {
				bys sasid: egen temp = max(`var')
				replace `var' = temp 
				drop temp
			}
			duplicates drop
		
			/* restrict to sufficiently large experiments */
			local QE_schs ""
			local N_QE_schs = 0
			foreach var of varlist riskset_* {
				local sch = substr("`var'",9,.)
				bys riskset_`sch': egen min = min(qual_`sch')
				bys riskset_`sch': egen max = max(qual_`sch')
				replace riskset_`sch' = 0 if min == max
				replace qual_`sch' = 0 if min == max
				drop min max
				qui count if riskset_`sch' > 0
				local count = r(N)
				disp "school `sch': N = `count' " _continue
				cap desc D_`sch'
				if `count' < `min_qesample' | _rc != 0  | "`sch'" == "4592" {
					disp "DROP"
					qui drop if riskset_`sch' > 0
					qui drop riskset_`sch' qual_`sch'
				}
				else {
					disp ""
					local QE_schs "`QE_schs' `sch'"
					local ++N_QE_schs
				}
			}		
			egen qe = rowmax(riskset_*)

			/* set up test data */
			preserve
			gen samp = "QE"
			if "`fdiff'" == "" {
				gen Y = c_bos_`VAM_t'rawsc6
			}
			if "`fdiff'" == "d" {
				gen Y = c_bos_`VAM_t'rawsc6 - bl_`VAM_t'				
			}
			foreach qe in `QE_schs' {
				gen RF_`qe' = qual_`qe'
			}
			foreach cont of varlist `QE_covs' {
				gen c_`cont'_RF = `cont'
			}
			foreach var of varlist riskset_* {
				qui tab `var', gen(`var'_RF_)
				drop `var'_RF_1
			}
			keep sasid samp Y RF_* c_*_RF riskset_*_RF_*
			tempfile samp_RF
			save `samp_RF', replace
			restore
			foreach vam in `VAM_schs' {
				preserve
				gen samp = "FS_`vam'"
				gen Y = D_`vam'
				summ Y
				if r(max) == 1 {
					foreach qe in `QE_schs' {
						gen FS_`qe'_`vam' = qual_`qe'
					}
					foreach cont of varlist `QE_covs' {
						gen c_`cont'_FS_`vam' = `cont'
					}
					foreach var of varlist riskset_* {
						qui tab `var', gen(`var'_FS_`vam'_)
						drop `var'_FS_`vam'_1
					}
					keep sasid samp Y FS_*_`vam' c_*_FS_`vam' riskset_*_FS_*_*
					tempfile samp_FS_`vam'
					save `samp_FS_`vam'', replace	
				}
				restore
			}
			use `samp_VAM', clear
			append using `samp_RF'
			foreach vam in `VAM_schs' {
				cap append using `samp_FS_`vam''
			}
			foreach var of varlist D_* `VAM_covs_m`model'' RF_* FS_*_* c_*_RF c_*_FS_* riskset_*_RF_* riskset_*_FS_*_* {
				replace `var' = 0 if `var' == .
			} 
			
			qui gen S_1 = samp == "VAM"
			ivreg2 Y D_* `VAM_covs_m`model'' RF_* FS_*_* c_*_RF c_*_FS_* riskset_*_RF_* riskset_*_FS_*_* S_*, cluster(sasid) partial(c_*_RF c_*_FS_* riskset_*_RF_* riskset_*_FS_*_* S_*)
	
			matrix ests = e(b)
			matrix vce  = e(V)
			cap mkdir "${tables}ests`rundate'"

			clear
			xsvmat vce, rowname(entry) fast
			local max = _N
			local entries ""
			forval i = 1/`max' {
				qui levelsof entry if _n==`i', local(temp)
				local temp `temp'
				if "`temp'" == "_cons" | substr("`temp'",1,2) == "o." {
					local temp "temp`i'"
				}
				local entries "`entries' `temp'"
				rename vce`i' `temp'
			}
			drop entry

			local j = 1
			foreach vam in `VAM_schs' {		
				rename D_`vam' VAM_`j'
				local ++j
			}
			local i = 1
			foreach sch1 in `VAM_schs' {
				cap rename RF_`sch1' RF_`i'
				local j = 1 
				foreach sch2 in `VAM_schs' {
					cap rename FS_`sch1'_`sch2' FS_`i'_`j'
					local ++j
				}
				local ++i
			}
			format * %20.0g
			outsheet using "${tables}ests`rundate'/VAM_vce_m`model'`fdiff'_test`test'_year`y'.csv", comma replace
			
			clear
			svmat ests, names(col)
			set obs 3
			local i = 1
			foreach sch1 in `VAM_schs' {	
				if strpos("`charters'","`sch1'")!=0 {
					replace D_`sch1' = 1 in 2
				}
				if strpos("`pilots'","`sch1'")!=0 {
					replace D_`sch1' = 1 in 3
				}
				rename D_`sch1' VAM_`i'
				cap rename RF_`sch1' RF_`i'
				local j = 1 
				foreach sch2 in `VAM_schs' {
					cap rename FS_`sch1'_`sch2' FS_`i'_`j'
					local ++j
				}
				local ++i
			}
			format * %20.0g
			outsheet using "${tables}ests`rundate'/VAM_ests_m`model'`fdiff'_test`test'_year`y'.csv" in 1, comma replace
			keep VAM_*
			drop if _n==1
			gen sect = _n
			reshape long VAM_, i(sect) j(sch)
			reshape wide VAM_, i(sch) j(sect)
			rename VAM_1 charter
			rename VAM_2 pilot
			replace charter = 0 if charter == .
			replace pilot = 0 if pilot == .
			outsheet using "${tables}ests`rundate'/VAM_sects_year`y'.csv", comma replace	
			}
		}
	}
}
if `encode_ests' == 1 {
	use "${cleandata}/VAM_sch_xwalk_year2007", clear
	gen year = 2007
	forval y = 2008/2014 {
		append using "${cleandata}/VAM_sch_xwalk_year`y'"
		replace year = `y' if year == .
	}
	reshape wide sch_num, i(sch) j(year)
	sort sch_num2007 sch_num2008 sch_num2009 sch_num2010 sch_num2011 sch_num2012 sch_num2013 sch_num2014
	gen id = _n
	order id
	drop sch
	outsheet using "${tables}ests`rundate'/VAM_sch_xwalk_by_year.csv", comma replace
}
