/******************************************************************/
/* Stata program for "Leveraging Lotteries for Value-Added, Fig 3 */
/*             Originally created September 14 2016 by PDH        */
/*                Last updated April 4 2017 by PDH                */
/******************************************************************/

clear all
cap log close
set more off
set trace off
set matsize 11000
set maxvar 32767
set seed 42
adopath + "Z:\ado\plus\"

/* directories */
global asgndata   "Z:/BPS/BPS assignment data/"
global cleandata  "Z:/BPS/Peter/BPSVA/Data/"
global distdata   "Z:/BPS/Peter/BPSVA/Data/location/distance files/"
global chartdata  "Z:/charters/MA Lotteries/all lotto data/school by year files/"
global programs   "Z:/BPS/Peter/BPSVA/Programs/"
global tables     "Z:/BPS/Peter/BPSVA/Tables/"
global graphs     "Z:/BPS/Peter/BPSVA/Graphs/"

/* timestamps */
local simsdate    "2015 01 19"
local charterdate "2014 08 10"

/* all baseline controls */
local all_covs    "bl_female bl_sch_female bl_black bl_sch_black bl_hisp bl_sch_hisp bl_white bl_sch_white bl_asian bl_sch_asian bl_sped bl_sch_sped bl_lep bl_sch_lep bl_foodfred bl_sch_foodfred bl_susp bl_sch_susp bl_truancy bl_sch_truancy bl_m bl_sch_m bl_e bl_sch_e y_*"

/* baseline VAM controls */
local VAM_covs_m1 "y_*"
local VAM_covs_m2 "bl_female bl_black bl_hisp bl_white bl_asian bl_sped bl_lep bl_foodfred bl_susp bl_truancy y_*"
local VAM_covs_m3 "bl_female bl_black bl_hisp bl_white bl_asian bl_sped bl_lep bl_foodfred bl_susp bl_truancy bl_y_2013 bl_m bl_m2 bl_m3 bl_e bl_e2 bl_e3 y_*"

/* baseline lottery controls */
local QE_covs_t0 "one"
local QE_covs_t1 "y_* bl_m bl_m2 bl_m3 bl_e bl_e2 bl_e3"
local QE_covs_t2 "y_* bl_m bl_m2 bl_m3 bl_e bl_e2 bl_e3 bl_female bl_black bl_hisp bl_white bl_asian bl_sped bl_lep bl_foodfred bl_susp bl_truancy bl_y_2013"

/* school sectors */
local charters "RoxP 4100205 4190305 MATCH 4280305 APR 4390050 4430205 4440205 UP 4490305 4570205 4590305 BosP 4630205 4750505 4810550 4900305"
local pilots   "1459 2360 2440 4160 4285 4331 4600 4661 4680 350077 350565"
local early_entry "350077 350080 4100205 4280305 4430205 4440205 4490305 4570205 4590305 4630205 4750505 4810550 4900305"

local min_qesample = 50
local ref_school   = 350691
local bs_reps      = 500
local bs_seed      = 42

local out = "c_bos_mrawsc6"

use "${cleandata}ms_obs_6th", clear
gen masscode = masscode6
merge m:1 masscode using "${cleandata}ms_fc_sc_schools_v2", keep(master match)
tostring masscode, replace
replace masscode = sch if _merge==3
drop _merge sch

local VAM_schs ""
qui levelsof masscode, local(schools)
foreach s in `schools' {
	if "`s'" != "`ref_school'" {		
		gen D_`s' = (masscode == "`s'")
		local VAM_schs "`VAM_schs' `s'"
	}
}

gen bl_me = 0.5*(bl_m+bl_e)
gen Y = `out'

merge 1:m sasid using "${cleandata}ms_fc_sc_exper_6th_v2", keep(master match)
gen qesamp = (_merge==3)
drop _merge	

/* restrict to sufficiently large experiments */
local QE_schs ""
local N_QE_schs = 0
foreach var of varlist riskset_* {
	local sch = substr("`var'",9,.)
	qui count if qesamp==1 & riskset_`sch' > 0
	local count = r(N)
	disp "school `sch': N = `count' " _continue
	cap desc D_`sch'*
	if `count' < `min_qesample' | _rc != 0  | "`sch'" == "4592"  {
		disp "DROP"
		qui replace qesamp = 0 if riskset_`sch' > 0
		qui drop riskset_`sch' qual_`sch'
	}
	else {
		gen samp_`sch' = qesamp & riskset_`sch' > 0
		local QE_schs "`QE_schs' `sch'"
		local ++N_QE_schs
		disp ""
	}
}

foreach var of varlist riskset_* qual_* qesamp samp_* {
	qui replace `var' = 0 if qesamp == 0
	qui bys sasid: egen temp = max(`var')
	qui replace `var' = temp
	drop temp
}	
gen temp = bl_m + bl_e
xtile bl_t_terc = temp, nq(3)
drop temp
keep if qesamp == 1
keep Y D_* qual_* `all_covs' bl_t_terc riskset_* sasid
duplicates drop

/* cov-stratified VAM */
egen T = group(bl_hisp bl_black bl_female bl_foodfred bl_sped bl_lep bl_t_terc)
foreach var of varlist D_* {
	qui levelsof T, local(types)
	foreach t in `types' {
		gen t_`var'_`t' = `var' * (T==`t')
		qui count if t_`var'_`t' == 1
		if r(N) < 1 {
			drop t_`var'_`t'
		}
	}
}
drop t_D_1459_40 y_8

gen wt = 1
matrix ests = J(`=1+`bs_reps'',`=3*`N_QE_schs'',.)

reg Y t_D_* bl_female bl_black bl_hisp bl_white bl_asian bl_sped bl_lep bl_foodfred bl_susp bl_truancy bl_y_2013 bl_m bl_m2 bl_m3 bl_e bl_e2 bl_e3 y_* [aw=wt]
matrix VAM = e(b)

local i = 1
local colnames ""
local AT ""
local NT ""
foreach sch in `QE_schs' {
	gen vam = 0
	qui levelsof T, local(types)
	foreach t in `types' {
		if colnumb(VAM,"t_D_`sch'_`t'")!=. {
			replace vam = VAM[1,colnumb(VAM,"t_D_`sch'_`t'")] if T==`t' & riskset_`sch'>0
		}
	}	
	qui areg qual_`sch' if riskset_`sch'>0 [aw=wt], absorb(riskset_`sch')
	predict pZ, xbd
	gen AT = (D_`sch'*(1-qual_`sch')/(1-pZ))*wt if pZ>0 & pZ<1
	gen NT = ((1-D_`sch')*qual_`sch'/pZ)*wt if pZ>0 & pZ<1
	gen C = (1 - AT - NT)*wt if pZ>0 & pZ<1
	reg vam [aw=AT]
		matrix ests[1,`i'] = _b[_cons]
	reg vam [aw=C]
		matrix ests[1,`i'+1] = _b[_cons]
		local AT "`AT' (AT_`sch' = C_`sch')"
	cap reg vam [aw=NT]
		if _rc == 0 | "`sch'" == "4033" {
			matrix ests[1,`i'+2] = _b[_cons]
			local NT "`NT' (NT_`sch' = C_`sch')"
		}
		else {
			reg vam [aw=AT]
				matrix ests[1,`i'+2] = _b[_cons]	
		}
	local colnames "`colnames' AT_`sch' C_`sch' NT_`sch'"
	local i = `i'+3
	drop vam pZ AT NT C
}
matrix colnames ests = `colnames'

clear
svmat ests, names(col)
keep if _n == 1
gen i = 1
reshape long AT_ C_ NT_, i(i) j(sch) str
rename AT_ AT
rename C_ C
rename NT_ NT
replace NT = . if NT == AT
gen charter = .
gen pilot = .
foreach sch in `QE_schs' {
	if strpos("`charters'","`sch'")!=0 {
		replace charter = 1 if sch == "`sch'"
	}
	if strpos("`pilots'","`sch'")!=0 {
		replace pilot = 1 if sch == "`sch'"
	}
}

scatter C AT if charter==. & pilot==., msymbol(diamond) || ///
	scatter C AT if pilot==1, msymbol(square) || ///
	scatter C AT if charter==1,  msymbol(triangle) || ///
	function y = x, lcolor(black) lpattern(dash) ra(-1 1) ylab(-1(0.5)1) xlab(-1(0.5)1) ///
	scheme(s1mono) legend(off) subtitle("A. Always-takers vs. compliers") ///
	xtitle("Always-takers") ytitle("Compliers") ///
	text(-0.95 0.75 "Joint {it:p}-value: 0.795")
	graph export ${tables}C_vs_AT_bytype.wmf, replace
	
scatter C NT if charter==. & pilot==., msymbol(diamond) || ///
	scatter C NT if pilot==1, msymbol(square) || ///
	scatter C NT if charter==1, msymbol(triangle) || ///
	function y = x, lcolor(black) lpattern(dash) ra(-1 1) ylab(-1(0.5)1) xlab(-1(0.5)1) ///
	scheme(s1mono) legend(off) subtitle("B. Never-takers vs. compliers")  ///
	xtitle("Never-takers") ytitle("Compliers")  ///
	text(-0.95 0.75 "Joint {it:p}-value: 0.332")
	graph export ${tables}C_vs_NT_bytype.wmf, replace


