/******************************************************************/
/*  Latergrade program for "Leveraging Lotteries for Value-Added  */
/*            Populates the "B" panels of Tables 3 & A2           */
/*   Also generates "first step" OLS, RF, and FS gr7/8 estimates  */
/*             Originally created April 10 2016 by PDH            */
/*                Last updated April 4 2017 by PDH                */
/******************************************************************/

clear all
cap log close
set more off
set trace off
set matsize 11000
set maxvar 32767
set seed 42
adopath + "Z:\ado\plus\"

/* directories */
global asgndata   "Z:/BPS/BPS assignment data/"
global cleandata  "Z:/BPS/Peter/BPSVA/Data/"
global distdata   "Z:/BPS/Peter/BPSVA/Data/location/distance files/"
global chartdata  "Z:/charters/MA Lotteries/all lotto data/school by year files/"
global programs   "Z:/BPS/Peter/BPSVA/Programs/"
global tables     "Z:/BPS/Peter/BPSVA/Tables/"
global graphs     "Z:/BPS/Peter/BPSVA/Graphs/"

/* timestamps */
local simsdate    "2015 01 19"
local charterdate "2014 08 10"
local rundate     "07052016"

/* all baseline controls */
local all_covs    "bl_female bl_sch_female bl_black bl_sch_black bl_hisp bl_sch_hisp bl_white bl_sch_white bl_asian bl_sch_asian bl_sped bl_sch_sped bl_lep bl_sch_lep bl_foodfred bl_sch_foodfred bl_susp bl_sch_susp bl_truancy bl_sch_truancy bl_m bl_sch_m bl_e bl_sch_e y_*"

/* baseline VAM controls */
local VAM_covs_m1 "y_*"
local VAM_covs_m2 "bl_female bl_black bl_hisp bl_white bl_asian bl_sped bl_lep bl_foodfred bl_susp bl_truancy y_*"
local VAM_covs_m3 "bl_female bl_black bl_hisp bl_white bl_asian bl_sped bl_lep bl_foodfred bl_susp bl_truancy bl_y_2013 bl_m bl_m2 bl_m3 bl_e bl_e2 bl_e3 y_*"

/* baseline lottery controls */
local QE_covs_t0 "one"
local QE_covs_t1 "y_* bl_m bl_m2 bl_m3 bl_e bl_e2 bl_e3"
local QE_covs_t2 "y_* bl_m bl_m2 bl_m3 bl_e bl_e2 bl_e3 bl_female bl_black bl_hisp bl_white bl_asian bl_sped bl_lep bl_foodfred bl_susp bl_truancy bl_y_2013"

/* school sectors */
local charters "RoxP 4100205 4190305 MATCH 4280305 APR 4390050 4430205 4440205 UP 4490305 4570205 4590305 BosP 4630205 4750505 4810550 4900305"
local pilots   "1459 2360 2440 4160 4285 4331 4600 4661 4680 350077 350565"
local early_entry "350077 350080 4100205 4280305 4430205 4440205 4490305 4570205 4590305 4630205 4750505 4810550 4900305"

/* program toggles  */
local ms_obs_setup  = 0
local forecast_test = 0
local overid_test   = 0
local fstep_ests    = 0

/* sampling options */
local min_enroll    = 25 
local min_qesample  = 50
local ref_school    = 350691
local nocharters    = 0 
local bs_reps       = 500
local bs_seed       = 42                              

/* model options */
local fd            = ""  
local VAM_t         = "m" 

/*****************************************/
/* ONE-STEP OVIERD TEST SUBROUTINE *******/
/*****************************************/
{
cap program drop rf_sandw
program rf_sandw, eclass
	syntax , weights(string) vam_covs(string) vam_ds(string) n_vam_schs(real) qe_covs(string) qe_schs(string) n_qe_schs(real) qe_test(string)
		preserve
		
		drop y_1			
		_rmcoll D_* `vam_covs' [aw=`weights'], forcedrop
		local VAM_Xs "`=r(varlist)'"
		reg Y `VAM_Xs' [aw=`weights']
		predict vamres, r
		predict VAM, xb
		foreach var of varlist D_* {
			local temp "`var'"
			local temp2: list VAM_Xs - temp
			local temp3: list VAM_Xs - temp2
			if "`temp3'" == "" {
				drop `var'
				local vam_ds: list vam_ds -  var
				local n_vam_schs = `n_vam_schs' - 1					
			}
		}

		matrix accum M_XX = `VAM_Xs' [aw=`weights']
		
		foreach var of varlist riskset_* {
			local sch = substr("`var'",9,.)
			local temp "`qe_schs'"
			local temp2: list temp - sch
			local temp3: list temp - temp2
			if "`temp3'" == "" {
				qui drop if riskset_`sch' > 0 & qesamp
				qui drop riskset_`sch' 
				cap drop qual_`sch'
			}
		}
		egen riskset = group(riskset_*) if qesamp
		qui tab riskset if qesamp, gen(d_)
	
		sort sasid

		/* Joint FC bias + Overid p-val */
		gen W = Y - VAM
		noisily areg W qual_* `qe_covs' [aw=`weights'], absorb(riskset) cluster(sasid)
		
		predict rfres, r
		matrix b = e(b)

		qui count if qesamp == 1
		matrix M = r(N)
		
		matrix M_XX = M_XX
		matrix accum ZD = qual_* `qe_covs' d_* `VAM_Xs' if qesamp == 1 [aw=`weights']
		matrix M_ZZ = ZD[1..`=rowsof(ZD)-rowsof(M_XX)',1..`=rowsof(ZD)-rowsof(M_XX)']
		matrix M_ZD = ZD[`=rowsof(ZD)-rowsof(M_XX)+1'..`=rowsof(ZD)',1..`=rowsof(ZD)-rowsof(M_XX)']'

		foreach var of varlist rfres qual_* `qe_covs' d_* {
			qui gen z_`var' = `var'
			qui replace z_`var' = 0 if qesamp == 0
		}
		local QE_Zs ""
		foreach z of varlist qual_* `qe_covs' d_*  {
			qui replace z_`z' = `z'*z_rfres
			local QE_Zs "`QE_Zs' z_`z'"
		}
		gen one2 = one
		foreach x of varlist `VAM_Xs' one2 {
			qui replace `x' = `x'*vamres
		}	
		
		sort sasid
		mata X = st_data(.,"`VAM_Xs' one2")
		mata M_XX = st_matrix("M_XX")
		mata M_ZD = st_matrix("M_ZD")	
		mata P = (M_ZD*invsym(M_XX)*X')' 

		local i = 1
		foreach z of varlist qual_* `qe_covs' d_*  {
			qui gen temp = .
			mata: st_store(.,"temp",P[.,`i'])			
			qui replace z_`z' = z_`z' - temp
			drop temp
			local ++i
		}
		
		matrix accum Q = `QE_Zs' [aw=`weights'], nocons
		matrix V = invsym(M_ZZ)*(Q)*invsym(M_ZZ)
		matrix b = b[1,1..`n_qe_schs']
		matrix V = V[1..`n_qe_schs',1..`n_qe_schs']		
		
		ereturn post b V
		test `qe_test'
		ereturn scalar F = r(chi2)
		restore

end	
}
/*****************************************/
/* SET UP MS OBSERVATIONAL SAMPLE ********/
/*****************************************/
if `ms_obs_setup'==1 {
	forval outgr = 7/8 {
		use "${cleandata}simsmcaswide `simsdate'", clear
		local outgr_l = `outgr' - 1
		keep if year`outgr' >= 2007 & year`outgr' <= 2014 & boston`outgr' == 1		
		
		/* Fix merges and splits */
		forval g = 5/`outgr' {
			replace masscode`g' = 350020 if masscode`g' == 350420 // Undo Curley Split
			replace masscode`g' = 350375 if masscode`g' == 350425 // Undo Edison Split
			replace masscode`g' = 350376 if masscode`g' == 350410 // Undo King Split
			replace masscode`g' = 350377 if masscode`g' == 350455 // Undo Lewis Split
			replace masscode`g' = 350378 if masscode`g' == 350381 // Undo Mildred Split
			replace masscode`g' = 350656 if masscode`g' == 350405 // Undo Umana Split
			replace masscode`g' = 350657 if masscode`g' == 350653 // Undo Umana Split	
		}

		/* drop small schools */
		gen one = 1
		bys masscode`outgr' year`outgr': egen kidcount = total(one)
		bys masscode`outgr': egen min_kidcount = min(kidcount)
		keep if min_kidcount >= `min_enroll'		
		
		foreach t in m e {
			gen bl_`t' = c_bos_`t'rawsc5
			qui summ bl_`t'
			replace bl_`t' = r(mean) if year`outgr' <= 2006 + (`outgr' - 6)
			gen int_`t' = c_bos_`t'rawsc`outgr_l'
			forval e = 2/3 {
				gen bl_`t'`e' = bl_`t'^`e'	
				gen int_`t'`e' = int_`t'^`e'	
			}
		}
		foreach v in female black hisp white asian sped lep foodfred {
			gen bl_`v'  = `v'5
			qui summ bl_`v'
			replace bl_`v' = r(mean) if year`outgr' <= 2006 + (`outgr' - 6)
			gen int_`v'  = `v'`outgr_l'
		}
		
		gen bl_susp = inschsusp5 + outschsusp5
		replace bl_susp = 0 if bl_susp < 0 
		qui summ bl_susp
		replace bl_susp = r(mean) if year`outgr' <= 2006 + (`outgr' - 6)
		gen int_susp = inschsusp`outgr_l' + outschsusp`outgr_l'
		replace int_susp = 0 if int_susp < 0
		gen bl_truancy = daystruant5
		replace bl_truancy = 0 if bl_truancy < 0 
		qui summ bl_truancy
		replace bl_truancy = r(mean) if year`outgr' <= 2006 + (`outgr' - 6)
		gen int_truancy = daystruant`outgr_l'
		replace int_truancy = 0 if int_truancy < 0
		qui tab year`outgr', gen(y_)
		gen bl_y_2013 = year5==2013
		gen int_y_2013 = year`outgr_l'==2013
		
		gen has_bl_int = 1
		foreach v of varlist `all_covs' {
			replace has_bl_int = 0 if `v' == .
		}
		keep if has_bl_int == 1 & c_bos_mrawsc`outgr'!=. & c_bos_erawsc`outgr'!=.	
		keep sasid masscode5 masscode6 masscode7 masscode8 c_bos_mrawsc`outgr' c_bos_erawsc`outgr' year`outgr' `all_covs'
	
	
		gen c_bos_merawsc`outgr' = c_bos_mrawsc`outgr' + c_bos_erawsc`outgr'
		compress
		save "${cleandata}ms_obs_`outgr'th", replace
	}
}
/*****************************************/
/* TESTS FOR FORECAST BIAS ***************/
/*****************************************/
if `forecast_test' == 1 {

	matrix fstep_test = J(20,10,.)
	
	/* Loop over grades */
	forval outgr = 6/8 {
		local col = `outgr' - 5
		local QE_covs `QE_covs_t2'

		use "${cleandata}ms_obs_`outgr'th", clear
		if `outgr' == 7 {
			drop if year7 == 2007
		}
		if `outgr' == 8 {
			drop if year8 == 2007 | year8 == 2008
		}
		if `outgr' == 8 {
			gen masscode = masscode8 
			drop masscode8
			drop if masscode == .
			merge m:1 masscode using "${cleandata}ms_fc_sc_schools_v2", keep(master match)
			tostring masscode, replace
			replace masscode = sch if _merge==3
			drop _merge sch
			rename masscode masscode8	
		}
		if `outgr' >= 7 {
			gen masscode = masscode7 
			drop masscode7
			drop if masscode == .
			merge m:1 masscode using "${cleandata}ms_fc_sc_schools_v2", keep(master match)
			tostring masscode, replace
			replace masscode = sch if _merge==3
			drop _merge sch
			rename masscode masscode7
		}
		gen masscode = masscode6
		drop masscode6
		merge m:1 masscode using "${cleandata}ms_fc_sc_schools_v2", keep(master match)
		tostring masscode, replace
		replace masscode = sch if _merge==3
		drop _merge sch
		rename masscode masscode6

		qui levelsof masscode`outgr', local(schools)
		foreach s in `schools' {
			if "`s'" != "`ref_school'" {
				if `outgr' == 6 {
					gen D_`s' = (masscode6 == "`s'")
				}
				if `outgr' == 7 {
					gen D_`s' = (masscode7 == "`s'") + (masscode6 == "`s'")
				}
				if `outgr' == 8 {
					gen D_`s' = (masscode8 == "`s'") + (masscode7 == "`s'") + (masscode6 == "`s'")
				}
			}
		}
		
		if "`fd'" == "" {
			gen Y`VAM_t' = c_bos_`VAM_t'rawsc`outgr'
		}
		if "`fd'" == "d" {	
			gen Y`VAM_t' = c_bos_`VAM_t'rawsc`outgr' - bl_`VAM_t'
		}

		gen one = 1
		drop y_1
		if "`fd'" == "" {
			ivreg2 Y`VAM_t' D_* `VAM_covs_m3'	
		}
		if "`fd'" == "d" {
			ivreg2 Y`VAM_t' D_* `VAM_covs_m2'	
		}
		predict vamres`VAM_t', r
		predict VAM`VAM_t', xb
		local VAM_Xs "`=e(inexog)'"
		matrix accum M_XX = `VAM_Xs'
		local N_VAM_covs = rowsof(M_XX)		
	
		merge 1:m sasid using "${cleandata}ms_fc_sc_exper_6th_v2", keep(master match)
		gen qesamp = (_merge==3)
		drop _merge	

		/* restrict to sufficiently large experiments */
		local QE_schs ""
		local QE_test ""
		local N_QE_schs = 0
		foreach var of varlist riskset_* {
			local sch = substr("`var'",9,.)
			qui count if qesamp==1 & riskset_`sch' > 0
			local count = r(N)
			disp "school `sch': N = `count' " _continue
			cap desc D_`sch'*
			if `count' < `min_qesample' | _rc != 0  | "`sch'" == "4592" | (`nocharters' == 1 & strpos("`charters'","`sch'") > 0) | ( "`outgr'" == "8" & "`sch'" == "2140") {
				disp "DROP"
				qui replace qesamp = 0 if riskset_`sch' > 0
				qui drop riskset_`sch' qual_`sch'
			}
			else {
				gen samp_`sch' = qesamp & riskset_`sch' > 0
				disp ""
				local QE_schs "`QE_schs' `sch'"
				local QE_test "`QE_test' qual_`sch'"
				local ++N_QE_schs
			}
		}
	
		foreach var of varlist riskset_* qual_* qesamp samp_* {
			qui replace `var' = 0 if qesamp == 0
			qui bys sasid: egen temp = max(`var')
			qui replace `var' = temp
			drop temp
		}	

		drop ch1 ch2 fc_hurdle sc_hurdle
		duplicates drop
		set seed 42
		gen tie = runiform()
		bys sasid (tie): gen keep = _n==1
		keep if keep
		drop keep

		foreach sch in `QE_schs' {
			qui tab riskset_`sch', gen(d_`sch'_)
		}
		drop masscode*
		tempfile analysis`outgr'
		save `analysis`outgr''
		sort sasid
		qui _rmcoll `QE_covs' d_*_* if qesamp==1, forcedrop
		local QE_Xs "`=r(varlist)'"
		
		/* FC bias test */
		ivreg2 Y (VAM = qual_*) `QE_Xs' if qesamp==1, cluster(sasid) partial(`QE_Xs')
			matrix fstep_test[5,`col'] = e(widstat)
		foreach var of varlist Y VAM qual_* {
			qui reg `var' `QE_Xs' if qesamp==1
			qui predict r_`var', r
		}
		ivregress 2sls r_Y (r_VAM = r_qual_*) if qesamp==1, nocons
		predict ivres, r
		matrix b = e(b)
		
		qui count
		matrix N = r(N)
		qui count if qesamp == 1
		matrix M = r(N)
		
		if "`fd'" == "d" {
			matrix accum ZD = r_qual_* D_* `VAM_covs_m2' if qesamp == 1
		}
		if "`fd'" == "" {
			matrix accum ZD = r_qual_* D_* `VAM_covs_m3' if qesamp == 1
		}		
		matrix M_ZZ = ZD[1..`N_QE_schs',1..`N_QE_schs']
		matrix M_ZD = ZD[`=`N_QE_schs'+1'..`=rowsof(ZD)',1..`N_QE_schs']'

		local temp = 0
		foreach var of varlist r_VAM {
			local ++temp
		}
		matrix accum EZY = r_Y r_VAM r_qual_* if qesamp == 1
		matrix M_ZY = EZY[`=`temp'+2'..`=rowsof(EZY)-1',1]
		matrix M_EZ = EZY[2..`=`temp'+1',`=`temp'+2'..`=rowsof(EZY)-1']	
		matrix M_XX = M_XX

		foreach var of varlist ivres r_qual_* {
			qui gen z_`var' = `var'
			qui replace z_`var' = 0 if qesamp == 0
		}
		local QE_Zs ""
		foreach z of varlist r_qual_* {
			qui replace z_`z' = z_`z'*z_ivres
			local QE_Zs "`QE_Zs' z_`z'"
		}
		gen one2 = one		
		foreach x of varlist `VAM_Xs' one2 {
			qui replace `x' = `x'*vamres
		}	
		
		sort sasid 
		cap sort sasid copy
		
		mata X = st_data(.,"`VAM_Xs' one2")
		mata M_XX = st_matrix("M_XX")
		mata M_ZD = st_matrix("M_ZD")
		mata P = (M_ZD*invsym(M_XX)*X')' 

		local i = 1
		foreach z of varlist r_qual_*  {
			qui gen temp = .
			mata: st_store(.,"temp",P[.,`i'])
			qui replace z_`z' = z_`z' - temp
			drop temp
			local ++i
		}

		matrix opaccum Q = `QE_Zs', group(sasid) opvar(one) nocons
		matrix V = (invsym(M_EZ*invsym(M_ZZ)*M_EZ')*M_EZ*(invsym(M_ZZ)*(Q)*invsym(M_ZZ))*M_EZ'*invsym(M_EZ*invsym(M_ZZ)*M_EZ'))
		matrix b = b[1,1]
		matrix V = V[1,1]		
		matrix colnames b = VAM
		matrix colnames V = VAM
		matrix rownames V = VAM
		ereturn post b V
		ereturn display
			matrix fstep_test[1,`col'] = _b[VAM]
			matrix fstep_test[2,`col'] = _se[VAM]			
		test VAM = 1
			matrix fstep_test[4,`col'] = r(p)			
		
		forval i = 1/2 {
			matrix beta_GMM = invsym(M_EZ*invsym(Q)*M_EZ')*M_EZ*invsym(Q)*M_ZY
			drop ivres
			gen ivres = r_Y - beta_GMM[1,1]*r_VAM

			drop z_*
			foreach var of varlist ivres r_qual_* {
				qui gen z_`var' = `var'
				qui replace z_`var' = 0 if qesamp==0
			}
			local QE_Zs ""
			foreach z of varlist r_qual_* {
				qui replace z_`z' = z_`z'*z_ivres
				local QE_Zs "`QE_Zs' z_`z'"
			}

			local i = 1
			foreach z of varlist r_qual_*  {
				qui gen temp = .
				mata: st_store(.,"temp",P[.,`i'])
				qui replace z_`z' = z_`z' - temp
				drop temp
				local ++i
			}
			matrix opaccum Q = `QE_Zs', group(sasid) opvar(one) nocons
			matrix V = (invsym(M_EZ*invsym(Q)*M_EZ'))
			matrix b = beta_GMM[1,1]
			matrix V = V[1,1]		
			matrix colnames b = VAM
			matrix colnames V = VAM
			matrix rownames V = VAM
			ereturn post b V
			ereturn display
				matrix fstep_test[11,`col'] = _b[VAM]
				matrix fstep_test[12,`col'] = _se[VAM]			
			test VAM = 1
				matrix fstep_test[13,`col'] = r(p)	
		}
		
		matrix opaccum Q = `QE_Zs' if qesamp==1, group(sasid) opvar(one) nocons
		matrix accum Ze = ivres r_qual_* if qesamp==1, nocons
		matrix Ze = Ze[2...,1]
		matrix J = Ze'*invsym(Q)*Ze
			matrix fstep_test[6,`col'] = chi2tail(`N_QE_schs'-1,J[1,1])	
	}

	use `analysis6', clear
	gen outgr = 6
	append using `analysis7'
	replace outgr = 7 if outgr == .
	append using `analysis8'
	replace outgr = 8 if outgr == .
	
	local ++col
	foreach var of varlist D_* {
		local sch = substr("`var'",3,.)
		replace `var' = 0 if `var' == .
		forval gr = 6/8 {
			gen `var'_`gr' = `var' * (outgr==`gr')
		}
		drop `var'
	}
	local stk_controls ""
	if "`fd'" == "" {
		foreach var of varlist `VAM_covs_m3' {
			forval gr = 6/8 {
				gen `var'_`gr' = `var' * (outgr==`gr')
				local stk_controls "`stk_controls' `var'_`gr'"
			}
		}
	}
	if "`fd'" == "d" {
		foreach var of varlist `VAM_covs_m2' {
			forval gr = 6/8 {
				gen `var'_`gr' = `var' * (outgr==`gr')
				local stk_controls "`stk_controls' `var'_`gr'"
			}
		}
	}
	gen outgr7 = outgr==7
	gen outgr8 = outgr==8
	matrix accum M_XX = D_* `stk_controls' outgr7 outgr8
	local N_VAM_covs = rowsof(M_XX)	
	
	drop d_*_*
	local QE_schs ""
	local QE_test ""
	local N_QE_schs = 0
	foreach var of varlist riskset_* {
		local sch = substr("`var'",9,.)
		forval gr = 6/8 {
			count if qual_`sch' != . & outgr == `gr'
			if r(N) > 0 {
				gen qual_`sch'_`gr' = qual_`sch' * (outgr==`gr')
				replace qual_`sch'_`gr' = 0 if qual_`sch'_`gr' == .
				local ++N_QE_schs
			}
		}
		drop qual_`sch'
		replace riskset_`sch' = 0 if riskset_`sch' == .
		egen temp = group(`var' outgr)
		qui tab temp, gen(d_`sch'_)
		drop temp
	}
	sort sasid
	qui _rmcoll `QE_covs' d_*_* if qesamp==1, forcedrop
	local QE_Xs "`=r(varlist)'"
		
	/* FC bias test */
	ivreg2 Y (VAM = qual_*) `QE_Xs' if qesamp==1, cluster(sasid) partial(`QE_Xs')
		matrix fstep_test[5,`col'] = e(widstat)

	foreach var of varlist Y VAM qual_* {
		qui reg `var' `QE_Xs' if qesamp==1
		qui predict r_`var', r
	}
	ivregress 2sls r_Y (r_VAM = r_qual_*) if qesamp==1, nocons
	predict ivres, r
	matrix b = e(b)
		
	qui count
	matrix N = r(N)
	qui count if qesamp == 1
	matrix M = r(N)
		
	matrix accum ZD = r_qual_* D_* `stk_controls' outgr7 outgr8 if qesamp == 1
	matrix M_ZZ = ZD[1..`N_QE_schs',1..`N_QE_schs']
	matrix M_ZD = ZD[`=`N_QE_schs'+1'..`=rowsof(ZD)',1..`N_QE_schs']'

	local temp = 0
	foreach var of varlist r_VAM {
		local ++temp
	}
	matrix accum EZY = r_Y r_VAM r_qual_* if qesamp == 1
	matrix M_ZY = EZY[`=`temp'+2'..`=rowsof(EZY)-1',1]
	matrix M_EZ = EZY[2..`=`temp'+1',`=`temp'+2'..`=rowsof(EZY)-1']	
	matrix M_XX = M_XX

	foreach var of varlist ivres r_qual_* {
		qui gen z_`var' = `var'
		qui replace z_`var' = 0 if qesamp == 0
	}
	local QE_Zs ""
	foreach z of varlist r_qual_* {
		qui replace z_`z' = z_`z'*z_ivres
		local QE_Zs "`QE_Zs' z_`z'"
	}
	gen one2 = one		
	foreach x of varlist D_* `stk_controls' outgr7 outgr8 one2 {
		qui replace `x' = `x'*vamres
	}	
	
	sort sasid 
	cap sort sasid copy
	
	mata X = st_data(.,"D_* `stk_controls' outgr7 outgr8 one2")
	mata M_XX = st_matrix("M_XX")
	mata M_ZD = st_matrix("M_ZD")
	mata P = (M_ZD*invsym(M_XX)*X')' 

	local i = 1
	foreach z of varlist r_qual_*  {
		qui gen temp = .
		mata: st_store(.,"temp",P[.,`i'])
		qui replace z_`z' = z_`z' - temp
		drop temp
		local ++i
	}

	matrix opaccum Q = `QE_Zs', group(sasid) opvar(one) nocons
	matrix V = (invsym(M_EZ*invsym(M_ZZ)*M_EZ')*M_EZ*(invsym(M_ZZ)*(Q)*invsym(M_ZZ))*M_EZ'*invsym(M_EZ*invsym(M_ZZ)*M_EZ'))
	matrix b = b[1,1]
	matrix V = V[1,1]		
	matrix colnames b = VAM
	matrix colnames V = VAM
	matrix rownames V = VAM
	ereturn post b V
	ereturn display
		matrix fstep_test[1,`col'] = _b[VAM]
		matrix fstep_test[2,`col'] = _se[VAM]			
	test VAM = 1
		matrix fstep_test[4,`col'] = r(p)			
	
	forval i = 1/2 {
		matrix beta_GMM = invsym(M_EZ*invsym(Q)*M_EZ')*M_EZ*invsym(Q)*M_ZY
		drop ivres
		gen ivres = r_Y - beta_GMM[1,1]*r_VAM

		drop z_*
		foreach var of varlist ivres r_qual_* {
			qui gen z_`var' = `var'
			qui replace z_`var' = 0 if qesamp==0
		}
		local QE_Zs ""
		foreach z of varlist r_qual_* {
			qui replace z_`z' = z_`z'*z_ivres
			local QE_Zs "`QE_Zs' z_`z'"
		}

		local i = 1
		foreach z of varlist r_qual_*  {
			qui gen temp = .
			mata: st_store(.,"temp",P[.,`i'])
			qui replace z_`z' = z_`z' - temp
			drop temp
			local ++i
		}
		matrix opaccum Q = `QE_Zs', group(sasid) opvar(one) nocons
		matrix V = (invsym(M_EZ*invsym(Q)*M_EZ'))
		matrix b = beta_GMM[1,1]
		matrix V = V[1,1]		
		matrix colnames b = VAM
		matrix colnames V = VAM
		matrix rownames V = VAM
		ereturn post b V
		ereturn display
			matrix fstep_test[11,`col'] = _b[VAM]
			matrix fstep_test[12,`col'] = _se[VAM]			
		test VAM = 1
			matrix fstep_test[13,`col'] = r(p)			
	}
	
	matrix opaccum Q = `QE_Zs' if qesamp==1, group(sasid) opvar(one) nocons
	matrix accum Ze = ivres r_qual_* if qesamp==1, nocons
	matrix Ze = Ze[2...,1]
	matrix J = Ze'*invsym(Q)*Ze
		matrix fstep_test[6,`col'] = chi2tail(`N_QE_schs'-1,J[1,1])			
	
	clear
	svmat fstep_test
	br
}
/*****************************************/
/* BOOTSTRAP FIRST STEP TESTS ************/
/*****************************************/
if `overid_test' == 1 {

	matrix fstep_bs_test = J(1,5,.)
	
	forval outgr = 6/8 {
		local col = `outgr' - 5
		use "${cleandata}ms_obs_`outgr'th", clear		
		if `outgr' == 7 {
			drop if year7 == 2007
		}
		if `outgr' == 8 {
			drop if year8 == 2007 | year8 == 2008
		}
		
		if `outgr' == 8 {
			gen masscode = masscode8 
			drop masscode8
			drop if masscode == .
			merge m:1 masscode using "${cleandata}ms_fc_sc_schools_v2", keep(master match)
			tostring masscode, replace
			replace masscode = sch if _merge==3
			drop _merge sch
			rename masscode masscode8	
		}
		if `outgr' >= 7 {
			gen masscode = masscode7 
			drop masscode7
			drop if masscode == .
			merge m:1 masscode using "${cleandata}ms_fc_sc_schools_v2", keep(master match)
			tostring masscode, replace
			replace masscode = sch if _merge==3
			drop _merge sch
			rename masscode masscode7
		}
		gen masscode = masscode6
		drop masscode6
		merge m:1 masscode using "${cleandata}ms_fc_sc_schools_v2", keep(master match)
		tostring masscode, replace
		replace masscode = sch if _merge==3
		drop _merge sch
		rename masscode masscode6

		local VAM_schs ""
		local VAM_Ds ""
		local N_VAM_schs = 0
		qui levelsof masscode`outgr', local(schools)
		foreach s in `schools' {
			if "`s'" != "`ref_school'" {
				if `outgr' == 6 {
					gen D_`s' = (masscode6 == "`s'")
				}
				if `outgr' == 7 {
					gen D_`s' = (masscode7 == "`s'") + (masscode6 == "`s'")
				}
				if `outgr' == 8 {
					gen D_`s' = (masscode8 == "`s'") + (masscode7 == "`s'") + (masscode6 == "`s'")
				}
				local VAM_Ds "`VAM_Ds' D_`s'"
				local ++N_VAM_schs
				local VAM_schs "`VAM_schs' `s'"
			}
		}

		if "`fd'" == "" {
			gen Y`VAM_t' = c_bos_`VAM_t'rawsc`outgr'
		}
		if "`fd'" == "d" {
			gen Y`VAM_t' = c_bos_`VAM_t'rawsc`outgr' - bl_`VAM_t'	
		}
		
		merge 1:m sasid using "${cleandata}ms_fc_sc_exper_6th_v2", keep(master match)
		gen qesamp = _merge==3
		drop _merge

		local QE_schs ""
		local N_QE_schs = 0
		foreach var of varlist riskset_* {
			local sch = substr("`var'",9,.)
			qui count if qesamp & riskset_`sch' > 0
			local count = r(N)
			disp "school `sch': N = `count' " _continue
			cap desc D_`sch'*
			if `count' < `min_qesample' | _rc != 0  | "`sch'" == "4592" | (`nocharters' == 1 & strpos("`charters'","`sch'") > 0) | ( "`outgr'" == "8" & "`sch'" == "2140") {
				disp "DROP"
				qui replace qesamp = 0 if riskset_`sch' > 0
				qui drop riskset_`sch' qual_`sch'
			}
			else {
				gen samp_`sch' = qesamp & riskset_`sch' > 0
				disp ""
				local QE_schs "`QE_schs' `sch'"
				local ++N_QE_schs
			}
		}			

	
		gen one = 1
		foreach var of varlist riskset_* qual_* qesamp {
			qui replace `var' = 0 if qesamp == 0
			qui bys sasid: egen temp = max(`var')
			qui replace `var' = temp
			drop temp
		}	
		drop ch1 ch2 fc_hurdle sc_hurdle
		duplicates drop
		set seed 42
		gen tie = runiform()
		bys sasid (tie): gen keep = _n==1
		keep if keep
		drop keep
		drop masscode*
		tempfile analysis`outgr'
		save `analysis`outgr''
		local QE_schs`outgr' "`QE_schs'"

		/* Bootstrap F stat */
		local QE_test ""
		foreach sch in `QE_schs' {
			if "`VAM_t'" != "m e" {
				local QE_test "`QE_test' (qual_`sch' = 0)"
			}
			if "`VAM_t'" == "m e" {
				local QE_test "`QE_test' (qual_`sch'_0 = 0) (qual_`sch'_1 = 0)"
			}			
		}	
		if "`fd'" == "" {
			rf_sandw, weights(one) vam_covs("`VAM_covs_m3'") vam_ds("`VAM_Ds'") n_vam_schs(`N_VAM_schs') qe_covs("`QE_covs_t2'") qe_schs("`QE_schs'") n_qe_schs(`N_QE_schs') qe_test("`QE_test'")
		}
		if "`fd'" == "d" {
			rf_sandw, weights(one) vam_covs("`VAM_covs_m2'") vam_ds("`VAM_Ds'") n_vam_schs(`N_VAM_schs') qe_covs("`QE_covs_t2'") qe_schs("`QE_schs'") n_qe_schs(`N_QE_schs') qe_test("`QE_test'")
		}
		local Fest = e(F)

		local QE_test ""
		foreach sch in `QE_schs' {
			if "`VAM_t'" != "m e" {
				local QE_test "`QE_test' (qual_`sch' = `=_b[qual_`sch']')"
			}
			if "`VAM_t'" == "m e" {
				local QE_test "`QE_test' (qual_`sch'_0 = `=_b[qual_`sch'_0]') (qual_`sch'_1 = `=_b[qual_`sch'_1]')"
			}			
		}

		set seed `bs_seed'
		matrix bs_F = J(`bs_reps',1,.)
		forval b = 1/`bs_reps' {
			disp "iter `b': " _continue
			qui {
			gen bs_wgt = rgamma(1,1)
			if "`fd'" == "" {
				rf_sandw, weights("bs_wgt") vam_covs("`VAM_covs_m3'") vam_ds("`VAM_Ds'") n_vam_schs(`N_VAM_schs') qe_covs("`QE_covs_t2'") qe_schs("`QE_schs'") n_qe_schs(`N_QE_schs') qe_test("`QE_test'")
			}
			if "`fd'" == "d" {
				rf_sandw, weights("bs_wgt") vam_covs("`VAM_covs_m2'") vam_ds("`VAM_Ds'") n_vam_schs(`N_VAM_schs') qe_covs("`QE_covs_t2'") qe_schs("`QE_schs'") n_qe_schs(`N_QE_schs') qe_test("`QE_test'")
			}
			drop bs_wgt
			matrix bs_F[`b',1] = e(F)
			}			
			disp "`=round(`=e(F)',0.01)'"
		}
		clear
		svmat bs_F
		count if bs_F1 > `Fest'
		matrix fstep_bs_test[1,`col'] = r(N)/`bs_reps'	

	}
	local ++col
	use `analysis6', clear
	gen outgr = 6
	append using `analysis7'
	replace outgr = 7 if outgr == .
	append using `analysis8'
	replace outgr = 8 if outgr == .
	
	local VAM_Ds ""
	local N_VAM_schs = 0
	foreach var of varlist D_* {
		local sch = substr("`var'",3,.)
		replace `var' = 0 if `var' == .
		forval gr = 6/8 {
			gen `var'_`gr' = `var' * (outgr==`gr')
			local VAM_Ds "`VAM_Ds' `var'_`gr'"
			local ++N_VAM_schs
		}
		drop `var'
	}
	local stk_controls ""
	if "`fd'" == "" {
		foreach var of varlist `VAM_covs_m3' {
			forval gr = 6/8 {
				gen `var'_`gr' = `var' * (outgr==`gr')
				local stk_controls "`stk_controls' `var'_`gr'"
			}
		}
	}
	if "`fd'" == "d" {
		foreach var of varlist `VAM_covs_m2' {
			forval gr = 6/8 {
				gen `var'_`gr' = `var' * (outgr==`gr')
				local stk_controls "`stk_controls' `var'_`gr'"
			}
		}
	}
	gen outgr7 = outgr==7
	gen outgr8 = outgr==8
	local stk_controls "`stk_controls' outgr7 outgr8"
	
	local QE_schs ""
	local N_QE_schs = 0
	local QE_test ""
	forval gr = 6/8 {
		foreach sch in `QE_schs`gr'' {
			gen qual_`sch'_`gr' = qual_`sch' * (outgr == `gr')
			replace qual_`sch'_`gr' = 0 if qual_`sch'_`gr' == .
			replace riskset_`sch' = 0 if riskset_`sch' == .
			egen temp = group(riskset_`sch' outgr)
			drop riskset_`sch'
			rename temp riskset_`sch'
			local QE_schs: list QE_schs | sch
			local ++N_QE_schs
			local QE_test "`QE_test' (qual_`sch'_`gr' = 0)"
		}
	}	
	foreach sch in `QE_schs' {
		drop qual_`sch'
	}
	rf_sandw, weights(one) vam_covs("`stk_controls'") vam_ds("`VAM_Ds'") n_vam_schs(`N_VAM_schs') qe_covs("`QE_covs_t2'") qe_schs("`QE_schs'") n_qe_schs(`N_QE_schs') qe_test("`QE_test'")
	local Fest = e(F)

	local QE_test ""
	forval gr = 6/8 {
		foreach sch in `QE_schs`gr'' {
			if "`VAM_t'" != "m e" {
				cap local QE_test "`QE_test' (qual_`sch'_`gr' = `=_b[qual_`sch'_`gr']')"
			}
			if "`VAM_t'" == "m e" {
				cap local QE_test "`QE_test' (qual_`sch'_`gr'_0 = `=_b[qual_`sch'_`gr'_0]') (qual_`sch'_`gr'_1 = `=_b[qual_`sch'_`gr'_1]')"
			}			
		}
	}

	set seed `bs_seed'
	matrix bs_F = J(`bs_reps',1,.)
	forval b = 1/`bs_reps' {
		disp "iter `b': " _continue
		qui {
		gen bs_wgt = rgamma(1,1)
		rf_sandw, weights("bs_wgt") vam_covs("`stk_controls'") vam_ds("`VAM_Ds'") n_vam_schs(`N_VAM_schs') qe_covs("`QE_covs_t2'") qe_schs("`QE_schs'") n_qe_schs(`N_QE_schs') qe_test("`QE_test'")
		drop bs_wgt
		matrix bs_F[`b',1] = e(F)
		}			
		disp "`=round(`=e(F)',0.01)'"
	}
	clear
	svmat bs_F
	count if bs_F1 > `Fest'
	matrix fstep_bs_test[1,`col'] = r(N)/`bs_reps'	

	clear
	svmat fstep_bs_test
	br

}
/*****************************************/
/* JOINT ESTIMATION OF FIRST STEP PARAMS */
/*****************************************/
if `fstep_ests' == 1 {

	/* Loop over grades */
	forval outgr = 6/8 {
		/* Loop over VAM models */
		forval model = 2/2 {
			/* Loop over QE tests */
			forval test = 2/2 { /* controls as in VA Model 3 */
			local QE_covs `QE_covs_t`test''

			use "${cleandata}ms_obs_`outgr'th", clear
			if `outgr' == 7 {
				drop if year7 == 2007
				drop y_1
			}
			if `outgr' == 8 {
				drop if year8 == 2007 | year8 == 2008
				drop y_1 y_2
			}
			
			gen masscode = masscode8 
			drop masscode8
			drop if masscode == .
			merge m:1 masscode using "${cleandata}ms_fc_sc_schools_v2", keep(master match)
			tostring masscode, replace
			replace masscode = sch if _merge==3
			drop _merge sch
			rename masscode masscode8			
			gen masscode = masscode7 
			drop masscode7
			drop if masscode == .
			merge m:1 masscode using "${cleandata}ms_fc_sc_schools_v2", keep(master match)
			tostring masscode, replace
			replace masscode = sch if _merge==3
			drop _merge sch
			rename masscode masscode7
			gen masscode = masscode6
			drop masscode6
			merge m:1 masscode using "${cleandata}ms_fc_sc_schools_v2", keep(master match)
			tostring masscode, replace
			replace masscode = sch if _merge==3
			drop _merge sch
			rename masscode masscode6
		
			local VAM_schs ""
			local N_VAM_schs = 0
			qui levelsof masscode`outgr', local(schools)
			foreach s in `schools' {
				if "`s'" != "`ref_school'" {
					if `outgr' == 7 {
						gen D_`s' = (masscode7 == "`s'")+(masscode6 == "`s'")
					}
					if `outgr' == 8 {
						gen D_`s' = (masscode8 == "`s'")+(masscode7 == "`s'")+(masscode6 == "`s'")
					}
					local VAM_schs "`VAM_schs' `s'"
					local ++N_VAM_schs
				}
			}

			preserve
			clear
			set obs `=`N_VAM_schs'+1'
			gen sch = "`ref_school'" if _n==1
			gen sch_num = 0 if _n==1
			forval j = 1/`N_VAM_schs' {
				replace sch = word("`VAM_schs'",`j') if _n==`j'+1
				replace sch_num = `j' if _n==`j'+1
			}
			save "${cleandata}VAM_sch_xwalk", replace
			restore
		
			preserve
			gen samp = "VAM"
			if "`fd'" == "" {
				gen Y = c_bos_`VAM_t'rawsc`outgr'
			}
			if "`fd'" == "d" {
				gen Y = c_bos_`VAM_t'rawsc`outgr' - bl_`VAM_t'				
			}
			keep sasid samp Y D_* `VAM_covs_m`model''
			tempfile samp_VAM
			save `samp_VAM', replace
			restore

			merge 1:m sasid using "${cleandata}ms_fc_sc_exper_6th_v2", keep(match)
			drop _merge

			/* restrict to sufficiently large experiments */
			local QE_schs ""
			local N_QE_schs = 0
			foreach var of varlist riskset_* {
				local sch = substr("`var'",9,.)
				qui count if riskset_`sch' > 0
				local count = r(N)
				disp "school `sch': N = `count' " _continue
				cap desc D_`sch'
				if `count' < `min_qesample' | _rc != 0  | "`sch'" == "4592" {
					disp "DROP"
					qui drop if riskset_`sch' > 0
					qui drop riskset_`sch' qual_`sch'
				}
				else {
					disp ""
					local QE_schs "`QE_schs' `sch'"
					local ++N_QE_schs
				}
			}

			/* set up test data */
			local copies = `N_VAM_schs' + 1
			local i = 1
			gen one = 1

			local QE_stk_cont ""
			foreach qe in `QE_schs' {
				preserve
				keep if riskset_`qe' > 0
				expand `copies'
				bys sasid: gen copy = _n - 1
				gen samp = "QE_`qe'_RF" if copy == 0
				if "`fd'" == "" {
					gen Y = c_bos_`VAM_t'rawsc`outgr' if copy == 0
				}
				if "`fd'" == "d" {
					gen Y = c_bos_`VAM_t'rawsc`outgr' - bl_`VAM_t' if copy == 0				
				}
				gen RF_`qe' = qual_`qe'*(copy==0)
				foreach cont of varlist `QE_covs' {
					gen c_`cont'_RF = `cont'*(copy == 0)
				}
				local j = 1
				foreach vam in `VAM_schs' {
					replace samp = "QE_`qe'_" + "`vam'" if copy == `j'
					replace Y = D_`vam' if copy == `j'
					gen FS_`qe'_`vam' = qual_`qe'*(copy == `j')
						foreach cont of varlist `QE_covs' {
							gen c_`cont'_FS`j' = `cont'*(copy == `j')
						}
					local ++j
				}
				egen riskset_for_`qe' = group(riskset_`qe' copy)
				keep sasid samp Y RF_`qe' FS_`qe'_* c_*_RF c_*_FS* riskset_for_`qe'
				tempfile samp_`qe'
				save `samp_`qe'', replace
				restore
			}

			use `samp_VAM', clear
			foreach qe in `QE_schs' {
				append using `samp_`qe''
			}
			foreach var of varlist D_* `VAM_covs_m`model'' RF_* FS_* c_*_RF c_*_FS* riskset_for_* {
				replace `var' = 0 if `var' == .
			}
			egen riskset = group(riskset_for_*)
			
			/* test regression    */
			if `outgr' == 7 {
				drop y_2
			}
			if `outgr' == 8 {
				drop y_3
			}
			areg Y D_* RF_* FS_*_* `VAM_covs_m`model'' c_*_RF c_*_FS*, absorb(riskset) cluster(sasid)
				matrix ests = e(b)
				matrix vce  = e(V)
			cap mkdir "${tables}ests`rundate'"

			clear
			xsvmat vce, rowname(entry) fast
			local max = _N
			local entries ""
			forval i = 1/`max' {
				qui levelsof entry if _n==`i', local(temp)
				local temp `temp'
				if "`temp'" == "_cons" | substr("`temp'",1,2) == "o." {
					local temp "temp`i'"
				}
				local entries "`entries' `temp'"
				rename vce`i' `temp'
			}
			cap drop bl_*
			cap drop y_* 
			cap drop temp*
			cap drop c_* 
			drop if substr(entry,1,3) == "bl_" | substr(entry,1,2) == "y_" | substr(entry,1,2) == "c_" | entry == "_cons"
			drop entry

			local j = 1

			foreach vam in `VAM_schs' {		
				rename D_`vam' VAM_`j'
				local ++j
			}

			local i = 1
			foreach sch1 in `VAM_schs' {
				cap rename RF_`sch1' RF_`i'
				local j = 1 
				foreach sch2 in `VAM_schs' {
					cap rename FS_`sch1'_`sch2' FS_`i'_`j'
					local ++j
				}
				local ++i
			}

			format * %20.0g
			egen tempM = rowmax(*)
			egen tempm = rowmin(*)
			drop if tempM == 0 & tempm == 0
			drop tempM tempm
			mkmat *, mat(V)
			matrix V = cholesky(V)
			outsheet using "${tables}ests`rundate'/VAM_vce_m`model'`fd'_test`test'_samp`vam_on_qe'.csv", comma replace
			
			clear
			svmat ests
			forval i = 1/`max' {
				local temp = word("`entries'",1)
				rename ests`i' `temp'
				local entries: list entries - temp
			}	
			cap drop bl_*
			cap drop y_* 
			cap drop temp*
			cap drop c_* 
			set obs 3
			local i = 1
			foreach sch1 in `VAM_schs' {	
				if strpos("`charters'","`sch1'")!=0 {
					replace D_`sch1' = 1 in 2
				}
				if strpos("`pilots'","`sch1'")!=0 {
					replace D_`sch1' = 1 in 3
				}
				rename D_`sch1' VAM_`i'
				cap rename RF_`sch1' RF_`i'
				local j = 1 
				foreach sch2 in `VAM_schs' {
					cap rename FS_`sch1'_`sch2' FS_`i'_`j'
					local ++j
				}
				local ++i
			}
			format * %20.0g
			outsheet using "${tables}ests`rundate'/VAM_ests_m`model'`fd'_test`test'_samp`vam_on_qe'.csv" in 1, comma replace

			keep VAM_*
			drop if _n==1
			gen sect = _n
			reshape long VAM_, i(sect) j(sch)
			reshape wide VAM_, i(sch) j(sect)
			rename VAM_1 charter
			rename VAM_2 pilot
			replace charter = 0 if charter == .
			replace pilot = 0 if pilot == .
			outsheet using "${tables}ests`rundate'/VAM_sects.csv", comma replace	
			}
		}
	}
}

