/******************************************************************/
/*     Tab5 program for "Leveraging Lotteries for Value-Added     */
/*             Originally created July 8 2016 by PDH              */
/*                Last updated April 4 2017 by PDH                */
/******************************************************************/

clear all
cap log close
set more off
set trace off
set matsize 11000
set maxvar 32767
set seed 42
adopath + "Z:\ado\plus\"

/* directories */
global asgndata   "Z:/BPS/BPS assignment data/"
global cleandata  "Z:/BPS/Peter/BPSVA/Data/"
global distdata   "Z:/BPS/Peter/BPSVA/Data/location/distance files/"
global chartdata  "Z:/charters/MA Lotteries/all lotto data/school by year files/"
global programs   "Z:/BPS/Peter/BPSVA/Programs/"
global tables     "Z:/BPS/Peter/BPSVA/Tables/"
global graphs     "Z:/BPS/Peter/BPSVA/Graphs/"

/* timestamps */
local simsdate    "2015 01 19"
local charterdate "2014 08 10"
local rundate     "07072015"

/* all baseline controls */
local all_covs    "bl_female bl_sch_female bl_black bl_sch_black bl_hisp bl_sch_hisp bl_white bl_sch_white bl_asian bl_sch_asian bl_sped bl_sch_sped bl_lep bl_sch_lep bl_foodfred bl_sch_foodfred bl_susp bl_sch_susp bl_truancy bl_sch_truancy bl_m bl_sch_m bl_e bl_sch_e y_*"

/* baseline VAM controls */
local VAM_covs_m1 "y_*"
local VAM_covs_m2 "bl_female bl_black bl_hisp bl_white bl_asian bl_sped bl_lep bl_foodfred bl_susp bl_truancy y_*"
local VAM_covs_m3 "bl_female bl_black bl_hisp bl_white bl_asian bl_sped bl_lep bl_foodfred bl_susp bl_truancy bl_y_2013 bl_m bl_m2 bl_m3 bl_e bl_e2 bl_e3 y_*"

/* baseline lottery controls */
local QE_covs_t0 "one"
local QE_covs_t1 "y_* bl_m bl_m2 bl_m3 bl_e bl_e2 bl_e3"
local QE_covs_t2 "y_* bl_m bl_m2 bl_m3 bl_e bl_e2 bl_e3 bl_female bl_black bl_hisp bl_white bl_asian bl_sped bl_lep bl_foodfred bl_susp bl_truancy bl_y_2013"

/* school sectors */
local charters "RoxP 4100205 4190305 MATCH 4280305 APR 4390050 4430205 4440205 UP 4490305 4570205 4590305 BosP 4630205 4750505 4810550 4900305"
local pilots   "1459 2360 2440 4160 4285 4331 4600 4661 4680 350077 350565"
local early_entry "350077 350080 4100205 4280305 4430205 4440205 4490305 4570205 4590305 4630205 4750505 4810550 4900305"

/* sampling options */
local min_enroll   = 25
local min_qesample = 50 
local ref_school   = 350691  

use "${cleandata}ms_obs_6th", clear
gen masscode = masscode6
merge m:1 masscode using "${cleandata}ms_fc_sc_schools_v2", keep(master match)
tostring masscode, replace
replace masscode = sch if _merge==3
drop _merge sch
	
local VAM_schs ""
local VAM_Ds ""
local N_VAM_schs = 0
qui levelsof masscode, local(schools)
foreach s in `schools' {
	if "`s'" != "`ref_school'" {
			gen D_`s' = (masscode == "`s'")
			local VAM_schs "`VAM_schs' `s'"
			local VAM_Ds "`VAM_Ds' D_`s'"
			local ++N_VAM_schs
	}
}
merge 1:m sasid using "${cleandata}ms_fc_sc_exper_6th_v2", keep(master match)
gen qesamp = (_merge==3)
drop _merge	

/* restrict to sufficiently large experiments */
local QE_schs ""
local QE_test ""
local N_QE_schs = 0
foreach var of varlist riskset_* {
	local sch = substr("`var'",9,.)
	qui count if qesamp==1 & riskset_`sch' > 0
	local count = r(N)
	disp "school `sch': N = `count' " _continue
	cap desc D_`sch'*
	if `count' < `min_qesample' | _rc != 0  | "`sch'" == "4592" {
		disp "DROP"
		qui replace qesamp = 0 if riskset_`sch' > 0
		qui drop riskset_`sch' qual_`sch'
	}
	else {
		gen samp_`sch' = qesamp & riskset_`sch' > 0
		disp ""
		local QE_schs "`QE_schs' `sch'"
		local QE_test "`QE_test' qual_`sch'"
		local ++N_QE_schs
	}
}

foreach var of varlist riskset_* qual_* qesamp samp_* {
	qui replace `var' = 0 if qesamp == 0
	qui bys sasid: egen temp = max(`var')
	qui replace `var' = temp
	drop temp
}	
drop ch1 ch2 fc_hurdle sc_hurdle
duplicates drop

matrix lotcounts = J(`N_VAM_schs',`=`N_QE_schs'+1',.)
local i = 1
foreach var1 of varlist D_* {
	local sch = substr("`var1'",3,.)
	local j = 1
	if strpos("`early_entry'","`sch'")!=0 {
		matrix lotcounts[`i',`j'] = 1
	}
	local ++j
	foreach var2 of varlist riskset_* {
		local sch = substr("`var2'",9,.)
		qui count if `var1' == 1 & riskset_`sch' > 0 & qual_`sch' == 0
		matrix lotcounts[`i',`j'] = r(N)
		local ++j
	}
	local ++i
}

clear 
insheet using "${cleandata}reorder_`rundate'.csv", clear
tempfile xwalk
save `xwalk'
clear
svmat lotcounts
gen oldsch = _n
rename lotcounts1 early_entry
replace early_entry = 0 if early_entry == .
merge 1:1 oldsch using `xwalk'
drop oldsch
sort newsch
mkmat lotcounts*, mat(counts)
keep newsch early_entry
tempfile early_entry
save `early_entry', replace

insheet using "${cleandata}VAM_vce_m2d_test2.csv", clear
mkmat *, mat(vce)
insheet using "${cleandata}VAM_ests_m2d_test2.csv", clear
xpose, clear varname
split _varname, parse("_")
destring _varname2, replace
destring _varname3, replace

summ _varname2 if _varname1 == "vam"
local J = r(max)
summ _varname2 if _varname1 == "rf"
local L = r(max)
gen n = _n

matrix test = J(`=`J'-`L'',5,0)
local row = 1
preserve
forval j = `=`L' + 1'/`J' {
	restore, preserve
	matrix test[`row',1] = `j'
	keep if _varname1 == "fs" & _varname3 == `j'
	forval l = 1/`L' {
		local temp = counts[`j',`l']
		if `temp' < 1 {
			drop if _varname2 == `l'
		}
		matrix test[`row',5] = test[`row',5] + `temp'
	}
	qui count
	local T = r(N)
	levelsof n, local(rows)
	mkmat v1, mat(testbeta)
	clear
	svmat vce
	gen n = _n
	local i = 1
	foreach var of varlist * {
		local temp1: list rows - i
		local temp2: list rows - temp1
		if "`temp2'" == "" {
			drop `var'
		}
		local ++i
	}
	qui count
	gen drop = 1
	foreach n in `rows' {
		replace drop = 0 if _n == `n'
	}
	drop if drop
	drop drop
	mkmat *, mat(testvce)
	matrix tmp = testbeta'*invsym(testvce)*testbeta
	local test = tmp[1,1]
	matrix test[`row',2] = `test'
	matrix test[`row',3] = `T'
	matrix test[`row',4] = chi2tail(`T',`test')
	local ++row
}
restore
insheet using "${cleandata}VAM_sects.csv", clear
rename sch test1
tempfile sects
save `sects'
clear
svmat test
merge 1:1 test1 using `sects', keep(match)
drop _merge
rename test1 newsch
merge 1:1 newsch using `early_entry', keep(match)
sort charter pilot early_entry newsch
br
