/******************************************************************/
/*    TabA1 program for "Leveraging Lotteries for Value-Added     */
/*            Originally created March 31 2016 by PDH             */
/*                 Last updated April 4 2017 by PDH               */
/******************************************************************/

clear all
cap log close
set more off
set trace off
set matsize 11000
set maxvar 32767
set seed 42
adopath + "Z:\ado\plus\"

/* directories */
global asgndata   "Z:/BPS/BPS assignment data/"
global cleandata  "Z:/BPS/Peter/BPSVA/Data/"
global distdata   "Z:/BPS/Peter/BPSVA/Data/location/distance files/"
global chartdata  "Z:/charters/MA Lotteries/all lotto data/school by year files/"
global programs   "Z:/BPS/Peter/BPSVA/Programs/"
global tables     "Z:/BPS/Peter/BPSVA/Tables/"
global graphs     "Z:/BPS/Peter/BPSVA/Graphs/"

local simsdate    "2015 01 19"

/* school sectors */
local charters "RoxP 4100205 4190305 MATCH 4280305 APR 4390050 4430205 4440205 UP 4490305 4570205 4590305 BosP 4630205 4750505 4810550 4900305"
local pilots   "1459 2360 2440 4160 4285 4331 4600 4661 4680 350077 350565"
local early_entry "350077 350080 4100205 4280305 4430205 4440205 4490305 4570205 4590305 4630205 4750505 4810550 4900305"

/* sampling options */
local min_enroll   = 25 
local min_qesample = 50 
local ref_school   = 350691                        

use "${cleandata}simsmcaswide `simsdate'", clear
bys masscode6 year6: egen kidcount = sum(1)
bys masscode6: egen min_kidcount   = min(kidcount)
gen in_samp = min_kidcount >= `min_enroll'
keep if boston5 == 1 & c_bos_mrawsc5!=. & c_bos_erawsc5!=. & boston6 == 1 & c_bos_mrawsc6!=.
keep sasid in_samp
compress
tempfile has_outcomes
save `has_outcomes'
use "${cleandata}ms_obs_6th", clear
gen masscode = masscode6
merge m:1 masscode using "${cleandata}ms_fc_sc_schools_v2", keep(master match)
tostring masscode, replace
replace masscode = sch if _merge==3
drop _merge sch
local VAM_schs ""
local N_VAM_schs = 0
qui levelsof masscode, local(schools)
foreach s in `schools' {
	if "`s'" != "`ref_school'" {
		gen D_`s' = (masscode == "`s'")
		local VAM_schs "`VAM_schs' `s'"
		local ++N_VAM_schs
	}
}
preserve
use "${cleandata}ms_fc_sc_exper_6th_v2", clear
set seed 42
gen temp = runiform()
bys sasid (temp): gen temp2 = _n==1
bys sasid yearapp: egen keep = max(temp2)
keep if keep
drop temp temp2 keep
tempfile exper_uniq
save `exper_uniq'
restore	
merge 1:m sasid using `exper_uniq', keep(master match)
gen qe = (_merge == 3)
drop _merge
local QE_schs ""
foreach var of varlist riskset_* {
	local sch = substr("`var'",9,.)
	qui count if riskset_`sch' > 0 & qe == 1
	disp "school `sch': N = `=r(N)' " _continue
	local N = r(N)
	cap desc D_`sch'
	if `N' < `min_qesample' | _rc!=0 {
		disp "DROP"
		qui replace qe = 0 if riskset_`sch' > 0
		qui replace gradeapp = . if riskset_`sch' > 0
		qui replace yearapp = . if riskset_`sch' > 0
		qui drop riskset_`sch' qual_`sch'
	}
	else {
		disp ""
		local QE_schs "`QE_schs' `sch'"
	}
}	
use "${cleandata}ms_fc_sc_exper_6th_v2", clear
set seed 42
gen temp = runiform()
bys sasid (temp): gen temp2 = _n==1
bys sasid yearapp: egen keep = max(temp2)
keep if keep
drop temp temp2 keep
foreach var of varlist riskset_* {
	local sch = substr("`var'",9,.)
	if strpos("`QE_schs'","`sch'") == 0 {
		qui drop riskset_`sch' qual_`sch'
	}
}
egen temp = rowmax(riskset_*)
drop if temp <= 0
drop temp
foreach var of varlist riskset_* qual_* {
	qui bys sasid: egen temp = max(`var')
	qui replace `var' = temp
	drop temp
}
keep sasid qual_* riskset_* 
duplicates drop

merge 1:1 sasid using `has_outcomes', keep(master match)
gen has_outcomes = _merge==3
replace in_samp = 0 if in_samp == .
drop _merge

local trad_list "`QE_schs'"
local chart_list ""
foreach sch in `charters' {
	cap summ qual_`sch'
	if _rc == 0 {
		local trad_list: list trad_list - sch
		local chart_list "`chart_list' `sch'"
	}
}
local pilot_list ""
foreach sch in `pilots' {
	cap summ qual_`sch'
	if _rc == 0 {
		local trad_list: list trad_list - sch
		local pilot_list "`pilot_list' `sch'"
	}
}
foreach sect in chart pilot trad {
	local `sect'_qual ""
	local `sect'_rs ""
	foreach sch in ``sect'_list' {
		local `sect'_qual "``sect'_qual' qual_`sch'"
		local `sect'_rs "``sect'_rs' riskset_`sch'"			
	}	
}
egen any_qual = rowmax(qual_*)
egen any_rs = group(riskset_*), m
egen temp = rowmax(riskset_*)
replace any_rs = 0 if temp <= 0 
drop temp
egen chart_qual = rowmax(`chart_qual')
egen chart_rs = group(`chart_rs'), m
egen temp = rowmax(`chart_rs')
replace chart_rs = 0 if temp <= 0
drop temp
egen pilot_qual = rowmax(`pilot_qual')
egen pilot_rs = group(`pilot_rs'), m
egen temp = rowmax(`pilot_rs')
replace pilot_rs = 0 if temp <= 0
drop temp
egen trad_qual = rowmax(`trad_qual')
egen trad_rs = group(`trad_rs'), m
egen temp = rowmax(`trad_rs')
replace trad_rs = 0 if temp <= 0
drop temp

matrix balance = J(50,50,.)
local row = 1
local col = 1
foreach sect in any trad pilot chart {	
	foreach out in has_outcomes in_samp {
		areg `out' `sect'_qual if `sect'_rs>0, r absorb(`sect'_rs) 
			matrix balance[`row',`col']   = _b[`sect'_qual]
			matrix balance[`row'+1,`col'] = _se[`sect'_qual]
			matrix balance[`row'+2,`col'] = e(N)
		local row = `row' + 2
	}
	local row = 1
	local ++col
}
clear
svmat balance
br	
