/******************************************************************/
/*    TabA6 program for "Leveraging Lotteries for Value-Added     */
/*            Originally created March 31 2016 by PDH             */
/*                 Last updated April 4 2017 by PDH               */
/******************************************************************/

clear all
cap log close
set more off
set trace off
set matsize 11000
set maxvar 32767
set seed 42
adopath + "Z:\ado\plus\"

/* directories */
global asgndata   "Z:/BPS/BPS assignment data/"
global cleandata  "Z:/BPS/Peter/BPSVA/Data/"
global distdata   "Z:/BPS/Peter/BPSVA/Data/location/distance files/"
global chartdata  "Z:/charters/MA Lotteries/all lotto data/school by year files/"
global programs   "Z:/BPS/Peter/BPSVA/Programs/"
global tables     "Z:/BPS/Peter/BPSVA/Tables/"
global graphs     "Z:/BPS/Peter/BPSVA/Graphs/"

local simsdate    "2015 01 19"
local charterdate "2014 08 10"

local QE_covs_t2 "y_* bl_m bl_m2 bl_m3 bl_e bl_e2 bl_e3 bl_female bl_black bl_hisp bl_white bl_asian bl_sped bl_lep bl_foodfred bl_susp bl_truancy bl_y_2013"
local charters "RoxP 4100205 4190305 MATCH 4280305 APR 4390050 4430205 4440205 UP 4490305 4570205 4590305 BosP 4630205 4750505 4810550 4900305"

/* sampling options */
local min_qesample = 50                           
local nocharters   = 0  

matrix fstep_test = J(20,10,.)

use "${cleandata}ms_obs_6th", clear
gen masscode = masscode6
merge m:1 masscode using "${cleandata}ms_fc_sc_schools_v2", keep(master match)
tostring masscode, replace
replace masscode = sch if _merge==3
drop _merge sch

qui levelsof masscode, local(schools)
foreach s in `schools' {
	if "`s'" != "`ref_school'" {
		gen D_`s' = (masscode == "`s'")
	}
}
gen Y = c_bos_mrawsc6

merge 1:m sasid using "${cleandata}ms_fc_sc_exper_6th_v2", keep(master match)
gen qesamp = (_merge==3)
drop _merge	

/* restrict to sufficiently large experiments */
local QE_schs ""
local QE_test ""
foreach var of varlist riskset_* {
	local sch = substr("`var'",9,.)
	qui count if qesamp==1 & riskset_`sch' > 0
	local count = r(N)
	disp "school `sch': N = `count' " _continue
	cap desc D_`sch'*
	if `count' < `min_qesample' | _rc != 0  | "`sch'" == "4592" | (`nocharters' == 1 & strpos("`charters'","`sch'") > 0) {
		disp "DROP"
		qui replace qesamp = 0 if riskset_`sch' > 0
		qui drop riskset_`sch' qual_`sch'
	}
	else {
		gen samp_`sch' = qesamp & riskset_`sch' > 0
		disp ""
		local QE_schs "`QE_schs' `sch'"
		local QE_test "`QE_test' qual_`sch'"
	}
}

foreach var of varlist riskset_* qual_* qesamp samp_* {
	qui replace `var' = 0 if qesamp == 0
	qui bys sasid: egen temp = max(`var')
	qui replace `var' = temp
	drop temp
}	
drop ch1 ch2 fc_hurdle sc_hurdle
duplicates drop

foreach sch in `QE_schs' {
	qui tab riskset_`sch', gen(d_`sch'_)
}

sort sasid
qui _rmcoll `QE_covs' d_*_* if qesamp==1, forcedrop
local QE_Xs "`=r(varlist)'"

gen sch = masscode
merge m:1 sch using "${cleandata}VAM_sch_xwalk_gr6"
drop _merge
rename sch_num old_id
merge m:1 old_id using "${cleandata}posteriors_stata", keepusing(beta_1 beta_3 beta_2d)
foreach m in 1 3 2d {
	replace beta_`m' = 0 if _merge == 1	
	gen res_`m' = Y - beta_`m'
}
drop _merge
keep Y beta_* res_* qual_* `QE_Xs' qesamp sasid
duplicates drop

/* Bias tests */
local col = 1
foreach m in 1 3 2d {

	ivreg2 Y (beta_`m' = qual_*) `QE_Xs' if qesamp==1, gmm2s cluster(sasid) partial(`QE_Xs')
		matrix fstep_test[1,`col'] = _b[beta_`m']
		matrix fstep_test[2,`col'] = _se[beta_`m']
		matrix fstep_test[3,`col'] = e(widstat)
		matrix fstep_test[5,`col'] = e(jp)
		test beta_`m' = 1
		matrix fstep_test[4,`col'] = r(p)		
	reg res_`m' qual_* `QE_Xs' if qesamp==1, r
		local QE_test_bs ""
		foreach sch in `QE_schs' {
			local QE_test_bs "`QE_test_bs' (qual_`sch' = `=_b[qual_`sch']')"		
		}
		test `QE_test'
		matrix fstep_test[6,`col'] = r(F)
		matrix fstep_test[7,`col'] = r(p)
	local ++col

}
clear
svmat fstep_test
br
