
*Chris Walters
*This program creates figures for the VAM project


	*********************************************************
	************** SET UP STATA *****************************
	**********************************************************
	
		*Basic setup
		clear
		set mem 500m
		set matsize 2000
		cap set more off
		cap log close
		cap set trace off
		
		*Directories
		local vam_folder = ""
		local current_folder="`vam_folder'/programs/revision_code/v71/estimation"
		
		
		*Switches
		local format_data=0
		local ols_std_devs=0
		local test_fig=0
		local post_fig=0
		local demand=0
		local rmse=0
		local fixed_random=0
		local char_regs=0
		
		*Data date
		local rundate     "07072015"
		
		
		/*****************************************/
		/* SECOND-STEP OVIERD TEST SUBROUTINE ****/
		/*****************************************/
		{
		cap program drop rf_delta
		program rf_delta, eclass
			syntax , vam_schs(string) n_vam_schs(real) qe_schs(string) n_qe_schs(real) fs_coefs(string) N_fs_coefs(real)

			matrix theta = e(b)'
			matrix Sigma = e(V)
			
			matrix gtop  = theta[`n_vam_schs'+1..`n_vam_schs'+`n_qe_schs',1]
			matrix Gtop1 = J(`n_qe_schs',`n_vam_schs',0)
			matrix Gtop2 = I(`n_qe_schs')
			matrix Gtop3 = J(`n_qe_schs',`n_fs_coefs',0)		
			
			matrix gbot  = J(`n_qe_schs',1,0)
			matrix Gbot1 = J(`n_qe_schs',`n_vam_schs',0)
			matrix Gbot2 = J(`n_qe_schs',`n_qe_schs',0)
			matrix Gbot3 = J(`n_qe_schs',`n_fs_coefs',0)
			
			local k = 1
			local qes "`qe_schs'"
			local vams "`vam_schs'"
			local title1 ""
			local title2 ""
			forval i = 1/`n_qe_schs' {	
				local qe  = word("`qes'",1)
				local title1 "`title1' QE_`qe'"
				local title2 "`title2' VAM_`qe'"
				forval j = 1/`n_vam_schs' {
					local vam = word("`vams'",1)
					local fs "`qe'_`vam'"
					local temp: list fs - fs_coefs
					if "`temp'" == "" {
						matrix gbot[`i',1] = gbot[`i',1] + _b[vam_`vam']*_b[fs_`qe'_`vam']
						cap matrix Gbot1[`i',`j'] = _b[fs_`qe'_`vam']
						matrix Gbot3[`i',`k'] = _b[vam_`vam']
						local ++k
					}
					local vams: list vams - vam
				}
				local vams "`vam_schs'"
				local qes: list qes - qe
			}	
			
			matrix g = [gtop\gbot]
			matrix Gtop  = [Gtop1,Gtop2,Gtop3]
			matrix Gbot  = [Gbot1,Gbot2,Gbot3]
			matrix G = [Gtop\Gbot]
			
			tempname b V
			matrix `b' = g'
			matrix colnames `b' = `title1' `title2'
			matrix `V'  = G*Sigma*G'
			matrix colnames `V' = `title1' `title2'
			matrix rownames `V' = `title1' `title2'
			ereturn post `b' `V'
		end	
		}


	*********************************************************
	************** CREATE FORMATTED DATA SET W/POSTERIORS ***
	*********************************************************
	
	
		if `format_data'==1 {
		
			*Oversubscription
				*Crosswalk
					insheet using "`current_folder'/reorder_`rundate'.csv", clear
					ren oldsch old_id
					ren newsch id
					save "`current_folder'/crosswalk.dta", replace

					*Demand
					use "`current_folder'/ms_demand.dta", clear
					ren sch_num old_id
					drop if old_id==0
					merge 1:1 old_id using "`current_folder'/crosswalk.dta"
					drop _merge
					tempfile oversub
					save "`oversub'"
		
			*Posterior estimates
			foreach m in "1" "3" "2d" {
			
				insheet using "`current_folder'/posteriors_results_`m'_sfx_pooled.csv", nonames comma clear
				gen school=_n
				tempfile model_`m'
				ren v1 alpha_hat_`m'
				ren v2 se_alpha_hat_`m'
				ren v3 alpha_post_`m'
				ren v4 beta_post_`m'
				ren v5 b_post_`m'
				ren v6 beta_`m'
				ren v7 b_`m'
				ren v8 charter
				ren v9 pilot
				foreach x in alpha_hat alpha_post beta_post b_post beta_`m' b_`m' {
					sum `x'
					replace `x' = `x'-r(mean)
				}
				save "`model_`m''"
				
			}

			merge 1:1 school using "`model_3'"
			drop _merge
			merge 1:1 school using "`model_1'"
			drop _merge
			gen id=_n
			merge 1:1 id using "`oversub'"
			keep if _merge==1 | _merge==3
			drop _merge
			gen lotto=_n<=28
			tempfile final
			save "`final'"
			
			
			*Merge in school demographic characteristics
			insheet using "`current_folder'/school_chars.csv", clear
			ren v1 hisp
			ren v2 black
			ren v3 white
			ren v4 female
			ren v5 foodfred
			ren v6 sped
			ren v7 lep
			ren v8 susp
			ren v9 truancy
			ren v10 baseline_m
			ren v11 baseline_e
			ren v12 id
			merge 1:1 id using "`final'"
			keep if _merge==3
			
			
			save "`current_folder'/posteriors_stata.dta", replace
			
		
		
		}
		
	*********************************************************
	************** STD DEVs of OLS VAM (FIGURE 1) ***********
	**********************************************************
	
		if `ols_std_devs'==1 {
		
			***Load data
			use "`current_folder'/posteriors_stata.dta", clear
		
			****Calculate std devs for each model
			gen sfx=_n==2
			
			foreach m in "1" "3" "2d" {
			
				qui gen sd_`m'=.
				qui sum alpha_hat_`m'
				local var=r(Var)
				qui gen v_alpha_hat_`m'=se_alpha_hat_`m'^2
				qui sum v_alpha_hat_`m'
				local sd_alpha_1 = sqrt(`var' - r(mean))
				disp "Model `m', No SFX: OLS VAM SD = `sd_alpha_1'"
				qui reg alpha_hat_`m' charter pilot
				qui predict alpha_res_`m', res
				qui sum alpha_res_`m'
				local var=r(Var)
				qui sum v_alpha_hat_`m'
				local sd_alpha_2 = sqrt(`var' - r(mean))
				disp "Model `m', SFX: OLS VAM SD = `sd_alpha_2'"
				replace sd_`m'=`sd_alpha_1' if sfx==0
				replace sd_`m'=`sd_alpha_2' if sfx==1
				
			
			}
			keep if _n<=2


			****Create graph
			graph bar sd_1 sd_3 sd_2d, over(sfx, relabel(1 "Total" 2 "Within sector")) ///
			ytitle("Std. dev. of OLS value-added") legend( label(1 "Uncontrolled") label(2 "Demographics") label(2 "Lagged score") label(3 "Gains") cols(3)) ///
			ylabel(, nogrid) ///
			scheme(s1mono)
			graph export vam_std.wmf, replace
			
		}
	
		/*****************************************/
		/* VIV FIGURE (FIGURE 2) ******************************/
		/*****************************************/
		if `test_fig' == 1 {
		
			matrix test_pvals = J(4,5,.)
			matrix test_ses = J(3,5,.)
			
			local row = 1
			local col = 1
			
			local va_coef_m1 = "0.396"
			local p_m1 = "<0.001"
			local va_coef_m2 = "0.645"
			local p_m2 = "<0.001"
			local va_coef_m2d = "0.950"
			local p_m2d = "<0.001"
			local va_coef_m3 = "0.864"
			local p_m3 = "<0.001"

			/* Loop over VAM models */
			foreach model in 3 2d {
				/* Loop over QE tests */
				forval test = 2/2 { /* most precise model */
				local QE_covs `QE_covs_t`test''

				insheet using "`vam_folder'/data/VAM_fstep_ests_`rundate'/VAM_vce_m`model'_test`test'.csv", clear
					mkmat *, matrix(vce)
					local c = r(N)
					local vars: colnames vce
					matrix rownames vce = `vars'
				insheet using "`vam_folder'/data/VAM_fstep_ests_`rundate'/VAM_ests_m`model'_test`test'.csv", clear
					mkmat *, matrix(ests)

				tempname b V
				matrix `b' = ests
				matrix `V' = vce
				ereturn post `b' `V'
				
				local VAM_schs ""
				local N_VAM_schs = 0
				foreach var of varlist vam_* {
					local temp = substr("`var'",5,.)
					local VAM_schs "`VAM_schs' `temp'"
					local ++N_VAM_schs
				}
				local QE_schs ""
				local N_QE_schs = 0
				local rftest ""
				foreach var of varlist rf_* {
					local temp = substr("`var'",4,.)
					local QE_schs "`QE_schs' `temp'"
					local ++N_QE_schs
					local rftest "`rftest' (QE_`temp' = VAM_`temp')"
				}	
				local FS_coefs ""
				local N_FS_coefs = 0
				foreach var of varlist fs_* {
					local temp = substr("`var'",4,.)
					local FS_coefs "`FS_coefs' `temp'"
					local ++N_FS_coefs
				}

				rf_delta, vam_schs(`VAM_schs') n_vam_schs(`N_VAM_schs') qe_schs(`QE_schs') n_qe_schs(`N_QE_schs') fs_coefs(`FS_coefs') n_fs_coefs(`N_FS_coefs')				

				matrix to_plot = J(`N_QE_schs',7,.)
				local i = 1
				foreach sch in `QE_schs' {	
					matrix to_plot[`i',1] = `sch'
					matrix to_plot[`i',2] = _b[VAM_`sch']
					matrix to_plot[`i',3] = _b[QE_`sch']
					
					lincom VAM_`sch' - QE_`sch'
					matrix to_plot[`i',4] = r(estimate)
					matrix to_plot[`i',5] = r(se)
					matrix to_plot[`i',6] = abs(r(estimate)/r(se))
					matrix to_plot[`i',7] = (1-normal(abs(r(estimate)/r(se))))*2
					local ++i

				}

				clear
				insheet using "`vam_folder'/data/VAM_fstep_ests_`rundate'/VAM_sects.csv", clear
				gen trad = 1 - charter - pilot
				tempfile sects
				save `sects', replace

				clear
				svmat to_plot
				rename to_plot1 sch
				rename to_plot2 VAM
				rename to_plot3 QE
				rename to_plot4 gap
				rename to_plot5 se
				rename to_plot6 t
				rename to_plot7 p
				
				merge 1:1 sch using `sects', keep(match)
				drop _merge
				
				/* VIV scatter*/
				local min = -0.4
				local gap = 0.2
				local max = 0.4	
				local order "QE VAM"
				local xtit "First stage effect on OLS value-added"
				local ytit "Reduced form effect on test scores"
				local slope = `va_coef_m`model''
				
				local new = _N + 1
				set obs `new' 
				gen eqline = `min' if VAM == .
				gen slopeline = `min'*`slope' if VAM == .
				local new = _N + 1
				set obs `new' 
				replace eqline = `max' if VAM == . & eqline == .	
				replace slopeline  = `max'*`slope' if VAM == . & slopeline == .		
				
				local xcl "black"
				local ycl "black"

				local addtext1 "Forecast coef.: `va_coef_m`model''"
				if "`model'" == "1" {
					local addtext1 "Forecast coef.:   `va_coef_m`model''"
				}
				local modeltitle "Uncontrolled"
				if "`model'" == "3" {
					local modeltitle "A. Lagged score"
				}
				if "`model'" == "2d" {
					local modeltitle "B. Gains"
				}
				graph twoway scatter `order' if t>=1.645 & trad & QE>`min' & QE<`max' & QE>`min' & QE<`max', color(gs6) msymbol(diamond) || ///
				   scatter `order' if t<1.645 & trad & QE>`min' & QE<`max' & QE>`min' & QE<`max', color(gs6) msymbol(diamond) mfcolor(white) || ///
				   scatter `order' if t>=1.645 & pilot & QE>`min' & QE<`max' & QE>`min' & QE<`max', color(gs10) msymbol(square) || ///
				   scatter `order' if t<1.645 & pilot & QE>`min' & QE<`max' & QE>`min' & QE<`max', color(gs10) msymbol(square) mfcolor(white) || ///
				   scatter `order' if t>=1.645 & charter & QE>`min' & QE<`max' & QE>`min' & QE<`max', color(gs8) msymbol(triangle) || ///
				   scatter `order' if t<1.645 & charter & QE>`min' & QE<`max' & QE>`min' & QE<`max', color(gs8) msymbol(triangle) mfcolor(white) || ///
				   line eqline eqline, lcolor(black) lwidth(thin) lpattern(dash)  || line slope eqline, lcolor(black) lwidth(thin) ///
				   scheme(s1mono) subtitle("`modeltitle'") legend(off) xtitle("`xtit'", color(`xcl')) ytitle("`ytit'", color(`ycl')) ///
				   xlab(`min'(`gap')`max', nolab) xmlab(`min'(`gap')`max', labcolor(black) labsize(medium)) ylab(`min'(`gap')`max', nolab) ymlab(`min'(`gap')`max', labsize(medium)) ///
				   text(-0.34 0.28 "`addtext1'") text(-0.38 0.27 "Omnibus {it:p}-val.: `p_m`model''", justification(right))			   
				graph export "VAM_scatter_m`model'_test`test'.wmf", replace	
				}
			}
	}

	*****************************************************
	*****  POSTERIOR FIGURE (FIGURE 4) *********************************
	*****************************************************
	
			if `post_fig'==1 {
		
			***Load data
			use "`current_folder'/posteriors_stata.dta", clear
			
			*****Make figs							
						
							
					*Lagged score model
					reg beta_3 alpha_post_3, r
					local reg = "0"+substr("`=round(_b[alpha_post_3],0.01)'",1,3)
					spearman beta_3 alpha_post_3
					local rank = "0"+substr("`=round(r(rho),0.01)'",1,3)	
			
					graph twoway scatter beta_3 alpha_post_3 if charter==1 & alpha_post_3<=1 & lotto, color(gs8) msymbol(T) ///
						|| scatter beta_3 alpha_post_3 if pilot==1 & alpha_post_3<=1 & lotto, color(gs10) msymbol(S) ///
						|| scatter beta_3 alpha_post_3 if charter==0 & pilot==0 & alpha_post_3<=1 & lotto, color(gs6) msymbol(D)  ///
						|| scatter beta_3 alpha_post_3 if charter==1 & alpha_post_3<=1 & !lotto, color(gs8) msymbol(T) mfcolor(white) ///
						|| scatter beta_3 alpha_post_3 if pilot==1 & alpha_post_3<=1 & !lotto, color(gs10) msymbol(S) mfcolor(white) ///
						|| scatter beta_3 alpha_post_3 if charter==0 & pilot==0 & alpha_post_3<=1 & !lotto, color(gs6) msymbol(D) mfcolor(white) ///
						|| lfit beta_3 alpha_post_3 if alpha_post_3<=1, lcolor(black) lpattern(dash) ///
						ytitle("Hybrid posterior mode") ///
						xtitle("Conventional posterior mean") subtitle("A. Lagged Score") ///
						scheme(s1mono) ///
						text(-0.45 0.4 "Reg. coef.: `reg'" "Rank corr.: `rank'") ///
							xlabel(-0.5(0.25)0.5) ylabel(-0.5(0.25)0.5) saving(posteriors3, replace) ///
							legend(off)
						graph export posteriors1.wmf, replace	

					*Gains model
					reg beta_2d alpha_post_2d, r
					local reg = "0"+substr("`=round(_b[alpha_post_2d],0.01)'",1,3)
					spearman beta_2d alpha_post_2d
					local rank = "0"+substr("`=round(r(rho),0.01)'",1,3)
				
					graph twoway scatter beta_2d alpha_post_2d if charter==1 & alpha_post_2d<=1 & lotto, color(gs8) msymbol(T)  ///
						|| scatter beta_2d alpha_post_2d if pilot==1 & alpha_post_2d<=1 & lotto, color(gs10) msymbol(S) ///
						|| scatter beta_2d alpha_post_2d if charter==0 & pilot==0 & alpha_post_2d<=1 & lotto, color(gs6) msymbol(D) ///
						|| scatter beta_2d alpha_post_2d if charter==1 & alpha_post_2d<=1 & !lotto, color(gs8) msymbol(T)  mfcolor(white) ///
						|| scatter beta_2d alpha_post_2d if pilot==1 & alpha_post_2d<=1 & !lotto, color(gs10) msymbol(S) mfcolor(white) ///
						|| scatter beta_2d alpha_post_2d if charter==0 & pilot==0 & alpha_post_2d<=1 & !lotto, color(gs6) msymbol(D) mfcolor(white) ///
						|| lfit beta_2d alpha_post_2d if alpha_post_2d<=1, lcolor(black) lpattern(dash)  ///
						ytitle("Hybrid posterior mode") ///
						xtitle("Conventional posterior mean") subtitle("B. Gains") ///
						scheme(s1mono) ///
						text(-0.45 0.4 "Reg. coef.: `reg'" "Rank corr.: `rank'") ///
							xlabel(-0.5(0.25)0.5) ylabel(-0.5(0.25)0.5) saving(posteriors2d, replace) ///
							legend(off)
						graph export posteriors2.wmf, replace
		}
	
	*****************************************************
	*****  OVERSUBSCRIPTION FIGURE (FIGURE 5) *********************************
	*****************************************************
	
		if `demand'==1 {
		
			***Load data
			use "`current_folder'/posteriors_stata.dta", clear
	
			***Setup
			replace oversub=log(oversub)
			keep if lotto==1 & oversub!=. & beta_3!=.
			reg oversub charter pilot, r
			predict oversub_resid, res
			reg b_3 charter pilot
			predict bias_resid, res
			reg bias_resid oversub_resid, r
			local reg = "0"+substr("`=round(_b[oversub_resid],0.01)'",1,3)
			local se = "0"+substr("`=round(_se[oversub_resid],0.01)'",1,3)
			
			
			***Make graph
			graph twoway scatter bias_resid oversub_resid if charter==0 & pilot==0, msymbol(D)  ///
			|| scatter bias_resid oversub_resid if pilot==1, msymbol(S) ///
			|| scatter bias_resid oversub_resid if charter==1, msymbol(T) ///
			|| lfit bias_resid oversub_resid, lpattern(dash) lcolor(black) ///
			legend(order(3 2 1) lab(3 "Charter") lab(2 "Pilot") lab(1 "Traditional") rows(1)) ///
			ylabel(, nogrid) legend(off) subtitle("A. Bias posteriors") ///
			ytitle("Bias posterior mode (residual)") xtitle("Oversubscription rate (residual)") ///
			scheme(s1mono) xlab(-1.5(0.5)1.5) ///
			text(-0.37 1.1 "Slope: -0.015 (0.056)")
			graph export demand.wmf, replace
		
		
		}
				
		
*****************************************************
*****  RMSE FIGURE (FIGURE 6) ***********************
*****************************************************

	if `rmse'==1 {

		*Load list of RMSE/Bias reported by matlab
		clear
		insheet using "`current_folder'/rmse_bias.csv"
		
		*Make graph
		graph bar bias rmse, over(type, gap(*.1) relabel(1 "Conv." 2 "Hybrid") label(labsize(small))) over(model, gap(*.3) relabel(1 "Uncontrolled" ///
		2 "Lagged score" 3 "Gains" 4 `" "Lagged score" "(sector effects)" "' 5 `" "Gains" "(sector effects)" "') label(labsize(small))) stack ///
		ytitle("Root mean squared error") bar(2, color(gs10) lcolor(gs10) lwidth(vthin)) bar(1, color(gs6) lcolor(gs6) lwidth(vthin)) ///
		legend(lab(2 "Variance") lab(1 "Bias") order(1 2)) graphregion(color(white)) ylabel(,nogrid)
		graph export rmse.wmf, replace
		

		
	}
		
	**********************************************************************
	***** FIXED V RANDOM FS (FIGURE A1) *********************************
	*********************************************************************
	
		
		if `fixed_random'==1 {
		
			***Load data
			use "`current_folder'/posteriors_stata.dta", clear
			
			***Make graph
			reg beta_3 beta_post_3
			correlate beta_3 beta_post_3
			graph twoway scatter beta_3 beta_post_3 if charter==0 & pilot==0, msymbol(D)  ///
			|| scatter beta_3 beta_post_3 if pilot==1, msymbol(S) ///
			|| scatter beta_3 beta_post_3 if charter==1, msymbol(T) ///
			|| lfit beta_3 beta_post_3, lpattern(dash) lcolor(black) ///
			legend(off) saving(fixed_random_1, replace) ///
			xlab(-0.5(0.25)0.5) ylabel(-0.5(0.25)0.5 ) ///
			ytitle("Estimated first stage") xtitle("Known first stage") subtitle("A. Value-added posterior") ///
			scheme(s1mono)  ///
			text(-0.45 0.38 "Correlation: 0.95")
			graph export fixed_random_1.wmf, replace		

			***Make graph
			reg b_3 b_post_3
			correlate b_3 b_post_3
			graph twoway scatter b_3 b_post_3 if charter==0 & pilot==0,  msymbol(D)  ///
			|| scatter b_3 b_post_3 if pilot==1, msymbol(S) ///
			|| scatter b_3 b_post_3 if charter==1, msymbol(T) ///
			|| lfit b_3 b_post_3, lpattern(dash) lcolor(black) ///
			legend(off) saving(fixed_random_2, replace) ///
			xlab(-0.5(0.25)0.5) ylabel(-0.5(0.25)0.5 ) ///
			ytitle("Estimated first stage") xtitle("Known first stage") subtitle("B. Bias posterior") ///
			scheme(s1mono) ///
			text(-0.45 0.38 "Correlation: 0.88")
			graph export fixed_random_2.wmf, replace
			
		}
		
	*********************************************************
	***** SCHOOL CHARACTERISTICS, VAM AND BIAS (TABLE 7)  **
	*********************************************************
	
		if `char_regs'==1 {
		
		
			*Load data
			use "`current_folder'/posteriors_stata.dta", clear
			
			*Data storage
			matrix T=J(100,10,.)
			local r=1
			local c=1
			
			*List of covariates
			local covs = "black hisp foodfred sped lep baseline_m baseline_e"
			
			*Regs without controls
			foreach v in beta b {
			
			
				foreach x of varlist `covs' {
				
					reg `v'_3 `x', r
					matrix T[`r',`c']=_b[`x']
					matrix T[`r'+1,`c']=_se[`x']
					local r=`r'+2
				}
				local r=1
				local ++c
			}
			local ++c
			
			*Regs with charter/pilot controls
			foreach v in beta b {
				foreach x of varlist `covs' {
					reg `v'_3 `x' charter pilot, r
					matrix T[`r',`c']=_b[`x']
					matrix T[`r'+1,`c']=_se[`x']
					local r=`r'+2
				}
				local r=1
				local ++c
			}

		clear
		svmat T
		browse

		
		}
	
	
	
		
