/* fill in missing data in year-by-year estimates */
local vers "05032016"

clear
set more off
set matsize 3000
set varabbrev off
cd "C:\Users\Hull\Dropbox (MIT)\VAM\data"

/* figure out which schools have lotteries and reorder */
forval y = 2007/2014 {
	insheet using ests`vers'\VAM_ests_m3_test2_year`y'.csv, clear
	keep rf_*
	gen year = `y'
	reshape long rf_, i(year) j(sch_num)
	drop rf_*
	tempfile year`y'
	save `year`y''
}
use `year2007'
forval y = 2008/2014 {
	append using `year`y''
}
tempfile lottos
save `lottos'
insheet using ests`vers'\VAM_sch_xwalk_by_year.csv, clear
reshape long sch_num, i(id) j(year)
merge m:1 year sch_num using `lottos'
gen lotto = (_merge == 3)
bys id: egen everlotto = max(lotto)
drop _merge lotto
reshape wide sch_num, i(id) j(year)
gsort -everlotto id
drop if id == 1 /* reference school */
gen new_id = _n
forval i = 1/50 {
	forval y = 2007/2014 {
		qui summ sch_num`y' if new_id == `i'
		local sch_`i'_`y' = r(mean)
	}
	qui summ everlotto if new_id == `i'
	local lotto_sch_`i' = r(mean)
}

/* sectors */
forval y = 2007/2014 {
	insheet using ests`vers'\VAM_sects_year`y'.csv, clear
	gen new_sch = .
	forval i = 1/50 {
		cap replace new_sch = `i' if sch == `sch_`i'_`y''
	}
	drop sch
	tempfile sects_`y'
	save `sects_`y''
}
use `sects_2007'
forval y = 2008/2014 {
	append using `sects_`y''
}
duplicates drop
sort new_sch
rename new_sch sch
cap mkdir ests_reformat`vers'
export delimited using ests_reformat`vers'\VAM_sects.csv, novarnames replace

/* ests */
forval y = 2007/2014 {
	foreach model in 3 2d {
		forval test = 2/2 {
			insheet using ests`vers'\VAM_ests_m`model'_test`test'_year`y'.csv, clear
			cap drop bl_*
			forval i = 1/50 {
				if `sch_`i'_`y'' != . {
					rename vam_`sch_`i'_`y'' n_vam_`i'
					cap rename rf_`sch_`i'_`y'' n_rf_`i'
					if _rc == 0 {
						forval j = 1/50 {
							if `sch_`j'_`y'' ! = . {
								rename fs_`sch_`i'_`y''_`sch_`j'_`y'' n_fs_`i'_`j'
							}
							else {
								gen n_fs_`i'_`j' = -999
							}
						}
					}
					else {
						gen n_rf_`i' = -999
						forval j = 1/50 {
							gen n_fs_`i'_`j' = -999
						}
					}
				}
				else {
					gen n_vam_`i' = -999
					gen n_rf_`i' = -999
					forval j = 1/50 {
						gen n_fs_`i'_`j' = -999
					}					
				}
			}	
			foreach var of varlist n_* {
				local stub = substr("`var'",3,.)
				rename `var' `stub'
			}
		order *, sequential
		order vam_* rf_* fs_*_*
		format * %20.0g
		export delimited using ests_reformat`vers'\VAM_ests_m`model'_test`test'_year`y'.csv, novarnames replace
		}
	}
	xpose, clear
	gen Q = (v1 != -999)
	keep Q
	xpose, clear
	export delimited using ests_reformat`vers'\Q_year`y'.csv, novarnames replace
}
/* vce */
forval y = 2007/2014 {
	foreach model in 3 2d {
		forval test = 2/2 {
			insheet using ests`vers'\VAM_vce_m`model'_test`test'_year`y'.csv, clear
			gen row = ""
			local i = 1
			foreach var of varlist * {
				replace row = "`var'" if _n==`i'
				local ++i
			}
			cap drop bl_*
			drop if substr(row,1,2)=="bl"
			forval i = 1/50 {
				if `sch_`i'_`y'' != . {
					rename vam_`sch_`i'_`y'' n_vam_`i'
					replace row = "n_vam_`i'" if row == "vam_`sch_`i'_`y''"
					cap rename rf_`sch_`i'_`y'' n_rf_`i'
					if _rc == 0 {
						replace row = "n_rf_`i'" if row == "rf_`sch_`i'_`y''"
						forval j = 1/50 {
							if `sch_`j'_`y'' ! = . {
								rename fs_`sch_`i'_`y''_`sch_`j'_`y'' n_fs_`i'_`j'
								replace row = "n_fs_`i'_`j'" if row == "fs_`sch_`i'_`y''_`sch_`j'_`y''"
							}
							else {
								gen n_fs_`i'_`j' = -999
								local nobs = _N + 1
								qui set obs `nobs'
								replace row = "n_fs_`i'_`j'" if row == ""
							}
						}
					}
					else {
						gen n_rf_`i' = -999
						local nobs = _N + 1
						qui set obs `nobs'
						replace row = "n_rf_`i'" if row == ""
						forval j = 1/50 {
							gen n_fs_`i'_`j' = -999
							local nobs = _N + 1
							qui set obs `nobs'
							replace row = "n_fs_`i'_`j'" if row == ""							
						}
					}
				}
				else {
					gen n_vam_`i' = -999
					local nobs = _N + 1
					qui set obs `nobs'
					replace row = "n_vam_`i'" if row == ""
					gen n_rf_`i' = -999
					local nobs = _N + 1
					qui set obs `nobs'
					replace row = "n_rf_`i'" if row == ""
					forval j = 1/50 {
						gen n_fs_`i'_`j' = -999
						local nobs = _N + 1
						qui set obs `nobs'
						replace row = "n_fs_`i'_`j'" if row == ""	
					}				
				}
			}	
			replace row = substr(row,3,.)
			foreach var of varlist n_* {
				local stub = substr("`var'",3,.)
				rename `var' `stub'
			}
			order *, sequential
			order vam_* rf_* fs_*_*
			gen seq = .
			local i = 1
			foreach var of varlist vam_* rf_* fs_*_* {
				replace seq = `i' if row == "`var'"
				local ++i
			}	
			sort seq
			drop row seq
			local i = 1
			foreach var of varlist * {
				qui summ `var'
				replace `var' = 0 if `var' == . | `var' == -999
				if r(min) == -999 {
					replace `var' = `i'/10000 in `i'
				}
				local ++i
			}
			mkmat *, mat(V)
			matrix test = cholesky(V)
			format * %20.0g
			export delimited using ests_reformat`vers'\VAM_vce_m`model'_test`test'_year`y'.csv, novarnames replace
		}
	}
}
